@@ -1,7 +1,6 @@
 use std::sync::{Arc, LazyLock};
 
 use tracing::trace;
-use url::Url;
 
 use cache::CredentialsCache;
 pub use credentials::Credentials;
@@ -9,6 +8,7 @@ pub use index::{AuthPolicy, Index, Indexes};
 pub use keyring::KeyringProvider;
 pub use middleware::AuthMiddleware;
 use realm::Realm;
+use uv_redacted::DisplaySafeUrl;
 
 mod cache;
 mod credentials;
@@ -28,7 +28,7 @@ pub(crate) static CREDENTIALS_CACHE: LazyLock<CredentialsCache> =
 /// Populate the global authentication store with credentials on a URL, if there are any.
 ///
 /// Returns `true` if the store was updated.
-pub fn store_credentials_from_url(url: &Url) -> bool {
+pub fn store_credentials_from_url(url: &DisplaySafeUrl) -> bool {
     if let Some(credentials) = Credentials::from_url(url) {
         trace!("Caching credentials for {url}");
         CREDENTIALS_CACHE.insert(url, Arc::new(credentials));
@@ -41,7 +41,7 @@ pub fn store_credentials_from_url(url: &Url) -> bool {
 /// Populate the global authentication store with credentials on a URL, if there are any.
 ///
 /// Returns `true` if the store was updated.
-pub fn store_credentials(url: &Url, credentials: Arc<Credentials>) {
+pub fn store_credentials(url: &DisplaySafeUrl, credentials: Arc<Credentials>) {
     trace!("Caching credentials for {url}");
     CREDENTIALS_CACHE.insert(url, credentials);
 }
