@@ -1,6 +1,7 @@
 use fs_err::File;
 use globset::{GlobSet, GlobSetBuilder};
 use itertools::Itertools;
+use rustc_hash::FxHashSet;
 use sha2::{Digest, Sha256};
 use std::io::{BufReader, Read, Write};
 use std::path::{Path, PathBuf};
@@ -17,8 +18,7 @@ use uv_warnings::warn_user_once;
 
 use crate::metadata::DEFAULT_EXCLUDES;
 use crate::{
-    BuildBackendSettings, DirectoryWriter, Error, FileList, ListWriter, PyProjectToml,
-    find_module_root, find_roots,
+    BuildBackendSettings, DirectoryWriter, Error, FileList, ListWriter, PyProjectToml, find_roots,
 };
 
 /// Build a wheel from the source tree and place it in the output directory.
@@ -124,60 +124,64 @@ fn write_wheel(
     let exclude_matcher = build_exclude_matcher(excludes)?;
 
     debug!("Adding content files to wheel");
-    let (src_root, module_root) = find_roots(
+    let (src_root, module_relative) = find_roots(
         source_tree,
         pyproject_toml,
         &settings.module_root,
         settings.module_name.as_ref(),
+        settings.namespace,
     )?;
 
     let mut files_visited = 0;
-    for entry in WalkDir::new(module_root)
-        .sort_by_file_name()
-        .into_iter()
-        .filter_entry(|entry| !exclude_matcher.is_match(entry.path()))
-    {
-        let entry = entry.map_err(|err| Error::WalkDir {
-            root: source_tree.to_path_buf(),
-            err,
-        })?;
+    let mut prefix_directories = FxHashSet::default();
+    for module_relative in module_relative {
+        // For convenience, have directories for the whole tree in the wheel
+        for ancestor in module_relative.ancestors().skip(1) {
+            if ancestor == Path::new("") {
+                continue;
+            }
+            // Avoid duplicate directories in the zip.
+            if prefix_directories.insert(ancestor.to_path_buf()) {
+                wheel_writer.write_directory(&ancestor.portable_display().to_string())?;
+            }
+        }
 
-        files_visited += 1;
-        if files_visited > 10000 {
-            warn_user_once!(
-                "Visited more than 10,000 files for wheel build. \
+        for entry in WalkDir::new(src_root.join(module_relative))
+            .sort_by_file_name()
+            .into_iter()
+            .filter_entry(|entry| !exclude_matcher.is_match(entry.path()))
+        {
+            let entry = entry.map_err(|err| Error::WalkDir {
+                root: source_tree.to_path_buf(),
+                err,
+            })?;
+
+            files_visited += 1;
+            if files_visited > 10000 {
+                warn_user_once!(
+                    "Visited more than 10,000 files for wheel build. \
                 Consider using more constrained includes or more excludes."
-            );
-        }
+                );
+            }
 
-        // We only want to take the module root, but since excludes start at the source tree root,
-        // we strip higher than we iterate.
-        let match_path = entry
-            .path()
-            .strip_prefix(source_tree)
-            .expect("walkdir starts with root");
-        let wheel_path = entry
-            .path()
-            .strip_prefix(&src_root)
-            .expect("walkdir starts with root");
-        if exclude_matcher.is_match(match_path) {
-            trace!("Excluding from module: `{}`", match_path.user_display());
-            continue;
-        }
-        let wheel_path = wheel_path.portable_display().to_string();
+            // We only want to take the module root, but since excludes start at the source tree root,
+            // we strip higher than we iterate.
+            let match_path = entry
+                .path()
+                .strip_prefix(source_tree)
+                .expect("walkdir starts with root");
+            let entry_path = entry
+                .path()
+                .strip_prefix(&src_root)
+                .expect("walkdir starts with root");
+            if exclude_matcher.is_match(match_path) {
+                trace!("Excluding from module: `{}`", match_path.user_display());
+                continue;
+            }
 
-        debug!("Adding to wheel: `{wheel_path}`");
-
-        if entry.file_type().is_dir() {
-            wheel_writer.write_directory(&wheel_path)?;
-        } else if entry.file_type().is_file() {
-            wheel_writer.write_file(&wheel_path, entry.path())?;
-        } else {
-            // TODO(konsti): We may want to support symlinks, there is support for installing them.
-            return Err(Error::UnsupportedFileType(
-                entry.path().to_path_buf(),
-                entry.file_type(),
-            ));
+            let entry_path = entry_path.portable_display().to_string();
+            debug!("Adding to wheel: {entry_path}");
+            wheel_writer.write_dir_entry(&entry, &entry_path)?;
         }
     }
     debug!("Visited {files_visited} files for wheel build");
@@ -267,16 +271,13 @@ pub fn build_editable(
     let mut wheel_writer = ZipDirectoryWriter::new_wheel(File::create(&wheel_path)?);
 
     debug!("Adding pth file to {}", wheel_path.user_display());
-    let src_root = source_tree.join(&settings.module_root);
-    if !src_root.starts_with(source_tree) {
-        return Err(Error::InvalidModuleRoot(settings.module_root.clone()));
-    }
-
     // Check that a module root exists in the directory we're linking from the `.pth` file
-    find_module_root(
-        &src_root,
+    let (src_root, _module_relative) = find_roots(
+        source_tree,
+        &pyproject_toml,
+        &settings.module_root,
         settings.module_name.as_ref(),
-        pyproject_toml.name(),
+        settings.namespace,
     )?;
 
     wheel_writer.write_bytes(
@@ -514,23 +515,12 @@ fn wheel_subdir_from_globs(
             continue;
         }
 
-        let relative_licenses = Path::new(target)
+        let license_path = Path::new(target)
             .join(relative)
             .portable_display()
             .to_string();
-
-        if entry.file_type().is_dir() {
-            wheel_writer.write_directory(&relative_licenses)?;
-        } else if entry.file_type().is_file() {
-            debug!("Adding {} file: `{}`", globs_field, relative.user_display());
-            wheel_writer.write_file(&relative_licenses, entry.path())?;
-        } else {
-            // TODO(konsti): We may want to support symlinks, there is support for installing them.
-            return Err(Error::UnsupportedFileType(
-                entry.path().to_path_buf(),
-                entry.file_type(),
-            ));
-        }
+        debug!("Adding for {}: `{}`", globs_field, relative.user_display());
+        wheel_writer.write_dir_entry(&entry, &license_path)?;
     }
     Ok(())
 }
