@@ -1,4 +1,3 @@
-use std::fmt::{Debug, Display, Formatter};
 use std::time::{Duration, SystemTime};
 use std::{borrow::Cow, path::Path};
 
@@ -100,44 +99,62 @@ where
     }
 }
 
-/// Either a cached client error or a (user specified) error from the callback
+/// Dispatch type: Either a cached client error or a (user specified) error from the callback
 pub enum CachedClientError<CallbackError: std::error::Error + 'static> {
-    Client(Error),
-    Callback(CallbackError),
+    Client {
+        retries: Option<u32>,
+        err: Error,
+    },
+    Callback {
+        retries: Option<u32>,
+        err: CallbackError,
+    },
 }
 
-impl<CallbackError: std::error::Error + 'static> Display for CachedClientError<CallbackError> {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+impl<CallbackError: std::error::Error + 'static> CachedClientError<CallbackError> {
+    /// Attach the number of retries to the error context.
+    ///
+    /// Adds to existing errors if any, in case different layers retried.
+    fn with_retries(self, retries: u32) -> Self {
         match self {
-            CachedClientError::Client(err) => write!(f, "{err}"),
-            CachedClientError::Callback(err) => write!(f, "{err}"),
+            CachedClientError::Client {
+                retries: existing_retries,
+                err,
+            } => CachedClientError::Client {
+                retries: Some(existing_retries.unwrap_or_default() + retries),
+                err,
+            },
+            CachedClientError::Callback {
+                retries: existing_retries,
+                err,
+            } => CachedClientError::Callback {
+                retries: Some(existing_retries.unwrap_or_default() + retries),
+                err,
+            },
         }
     }
-}
 
-impl<CallbackError: std::error::Error + 'static> Debug for CachedClientError<CallbackError> {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+    fn retries(&self) -> Option<u32> {
         match self {
-            CachedClientError::Client(err) => write!(f, "{err:?}"),
-            CachedClientError::Callback(err) => write!(f, "{err:?}"),
+            CachedClientError::Client { retries, .. } => *retries,
+            CachedClientError::Callback { retries, .. } => *retries,
         }
     }
-}
 
-impl<CallbackError: std::error::Error + 'static> std::error::Error
-    for CachedClientError<CallbackError>
-{
-    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+    fn error(&self) -> &dyn std::error::Error {
         match self {
-            CachedClientError::Client(err) => Some(err),
-            CachedClientError::Callback(err) => Some(err),
+            CachedClientError::Client { err, .. } => err,
+            CachedClientError::Callback { err, .. } => err,
         }
     }
 }
 
 impl<CallbackError: std::error::Error + 'static> From<Error> for CachedClientError<CallbackError> {
     fn from(error: Error) -> Self {
-        Self::Client(error)
+        Self::Client {
+            retries: None,
+            err: error,
+        }
     }
 }
 
@@ -145,30 +162,52 @@ impl<CallbackError: std::error::Error + 'static> From<ErrorKind>
     for CachedClientError<CallbackError>
 {
     fn from(error: ErrorKind) -> Self {
-        Self::Client(error.into())
+        Self::Client {
+            retries: None,
+            err: error.into(),
+        }
     }
 }
 
 impl<E: Into<Self> + std::error::Error + 'static> From<CachedClientError<E>> for Error {
+    /// Attach retry error context, if there were retries.
     fn from(error: CachedClientError<E>) -> Self {
         match error {
-            CachedClientError::Client(error) => error,
-            CachedClientError::Callback(error) => error.into(),
+            CachedClientError::Client {
+                retries: Some(retries),
+                err,
+            } => ErrorKind::RequestWithRetries {
+                source: Box::new(err.into_kind()),
+                retries,
+            }
+            .into(),
+            CachedClientError::Client { retries: None, err } => err,
+            CachedClientError::Callback {
+                retries: Some(retries),
+                err,
+            } => ErrorKind::RequestWithRetries {
+                source: Box::new(err.into().into_kind()),
+                retries,
+            }
+            .into(),
+            CachedClientError::Callback { retries: None, err } => err.into(),
         }
     }
 }
 
 #[derive(Debug, Clone, Copy)]
-pub enum CacheControl {
+pub enum CacheControl<'a> {
     /// Respect the `cache-control` header from the response.
     None,
     /// Apply `max-age=0, must-revalidate` to the request.
     MustRevalidate,
     /// Allow the client to return stale responses.
     AllowStale,
+    /// Override the cache control header with a custom value.
+    Override(&'a str),
 }
 
-impl From<Freshness> for CacheControl {
+impl From<Freshness> for CacheControl<'_> {
     fn from(value: Freshness) -> Self {
         match value {
             Freshness::Fresh => Self::None,
@@ -222,7 +261,7 @@ impl CachedClient {
         &self,
         req: Request,
         cache_entry: &CacheEntry,
-        cache_control: CacheControl,
+        cache_control: CacheControl<'_>,
         response_callback: Callback,
     ) -> Result<Payload, CachedClientError<CallBackError>> {
         let payload = self
@@ -255,7 +294,7 @@ impl CachedClient {
         &self,
         req: Request,
         cache_entry: &CacheEntry,
-        cache_control: CacheControl,
+        cache_control: CacheControl<'_>,
         response_callback: Callback,
     ) -> Result<Payload::Target, CachedClientError<CallBackError>> {
         let fresh_req = req.try_clone().expect("HTTP request must be cloneable");
@@ -385,7 +424,7 @@ impl CachedClient {
         let data = response_callback(response)
             .boxed_local()
             .await
-            .map_err(|err| CachedClientError::Callback(err))?;
+            .map_err(|err| CachedClientError::Callback { retries: None, err })?;
         let Some(cache_policy) = cache_policy else {
             return Ok(data.into_target());
         };
@@ -432,7 +471,7 @@ impl CachedClient {
     async fn send_cached(
         &self,
         mut req: Request,
-        cache_control: CacheControl,
+        cache_control: CacheControl<'_>,
         cached: DataWithCachePolicy,
     ) -> Result<CachedResponse, Error> {
         // Apply the cache control header, if necessary.
@@ -444,6 +483,13 @@ impl CachedClient {
                     http::HeaderValue::from_static("no-cache"),
                 );
             }
+            CacheControl::Override(value) => {
+                req.headers_mut().insert(
+                    http::header::CACHE_CONTROL,
+                    http::HeaderValue::from_str(value)
+                        .map_err(|_| ErrorKind::InvalidCacheControl(value.to_string()))?,
+                );
+            }
         }
         Ok(match cached.cache_policy.before_request(&mut req) {
             BeforeRequest::Fresh => {
@@ -451,7 +497,7 @@ impl CachedClient {
                 CachedResponse::FreshCache(cached)
             }
             BeforeRequest::Stale(new_cache_policy_builder) => match cache_control {
-                CacheControl::None | CacheControl::MustRevalidate => {
+                CacheControl::None | CacheControl::MustRevalidate | CacheControl::Override(_) => {
                     debug!("Found stale response for: {}", req.url());
                     self.send_cached_handle_stale(req, cached, new_cache_policy_builder)
                         .await?
@@ -486,7 +532,6 @@ impl CachedClient {
         debug!("Sending revalidation request for: {url}");
         let response = self
             .0
-            .for_host(&url)
             .execute(req)
             .instrument(info_span!("revalidation_request", url = url.as_str()))
             .await
@@ -527,12 +572,23 @@ impl CachedClient {
         let cache_policy_builder = CachePolicyBuilder::new(&req);
         let response = self
             .0
-            .for_host(&url)
             .execute(req)
             .await
-            .map_err(|err| ErrorKind::from_reqwest_middleware(url.clone(), err))?
-            .error_for_status()
-            .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
+            .map_err(|err| ErrorKind::from_reqwest_middleware(url.clone(), err))?;
+
+        let retry_count = response
+            .extensions()
+            .get::<reqwest_retry::RetryCount>()
+            .map(|retries| retries.value());
+
+        if let Err(status_error) = response.error_for_status_ref() {
+            return Err(CachedClientError::<Error>::Client {
+                retries: retry_count,
+                err: ErrorKind::from_reqwest(url, status_error).into(),
+            }
+            .into());
+        }
+
         let cache_policy = cache_policy_builder.build(&response);
         let cache_policy = if cache_policy.to_archived().is_storable() {
             Some(Box::new(cache_policy))
@@ -552,7 +608,7 @@ impl CachedClient {
         &self,
         req: Request,
         cache_entry: &CacheEntry,
-        cache_control: CacheControl,
+        cache_control: CacheControl<'_>,
         response_callback: Callback,
     ) -> Result<Payload, CachedClientError<CallBackError>> {
         let payload = self
@@ -576,10 +632,10 @@ impl CachedClient {
         &self,
         req: Request,
         cache_entry: &CacheEntry,
-        cache_control: CacheControl,
+        cache_control: CacheControl<'_>,
         response_callback: Callback,
     ) -> Result<Payload::Target, CachedClientError<CallBackError>> {
-        let mut n_past_retries = 0;
+        let mut past_retries = 0;
         let start_time = SystemTime::now();
         let retry_policy = self.uncached().retry_policy();
         loop {
@@ -587,11 +643,20 @@ impl CachedClient {
             let result = self
                 .get_cacheable(fresh_req, cache_entry, cache_control, &response_callback)
                 .await;
+
+            // Check if the middleware already performed retries
+            let middleware_retries = match &result {
+                Err(err) => err.retries().unwrap_or_default(),
+                Ok(_) => 0,
+            };
+
             if result
                 .as_ref()
-                .is_err_and(|err| is_extended_transient_error(err))
+                .is_err_and(|err| is_extended_transient_error(err.error()))
             {
-                let retry_decision = retry_policy.should_retry(start_time, n_past_retries);
+                // If middleware already retried, consider that in our retry budget
+                let total_retries = past_retries + middleware_retries;
+                let retry_decision = retry_policy.should_retry(start_time, total_retries);
                 if let reqwest_retry::RetryDecision::Retry { execute_after } = retry_decision {
                     debug!(
                         "Transient failure while handling response from {}; retrying...",
@@ -601,10 +666,15 @@ impl CachedClient {
                         .duration_since(SystemTime::now())
                         .unwrap_or_else(|_| Duration::default());
                     tokio::time::sleep(duration).await;
-                    n_past_retries += 1;
+                    past_retries += 1;
                     continue;
                 }
             }
+
+            if past_retries > 0 {
+                return result.map_err(|err| err.with_retries(past_retries));
+            }
+
             return result;
         }
     }
@@ -622,7 +692,7 @@ impl CachedClient {
         cache_entry: &CacheEntry,
         response_callback: Callback,
     ) -> Result<Payload, CachedClientError<CallBackError>> {
-        let mut n_past_retries = 0;
+        let mut past_retries = 0;
         let start_time = SystemTime::now();
         let retry_policy = self.uncached().retry_policy();
         loop {
@@ -630,12 +700,20 @@ impl CachedClient {
             let result = self
                 .skip_cache(fresh_req, cache_entry, &response_callback)
                 .await;
+
+            // Check if the middleware already performed retries
+            let middleware_retries = match &result {
+                Err(err) => err.retries().unwrap_or_default(),
+                _ => 0,
+            };
+
             if result
                 .as_ref()
                 .err()
-                .is_some_and(|err| is_extended_transient_error(err))
+                .is_some_and(|err| is_extended_transient_error(err.error()))
             {
-                let retry_decision = retry_policy.should_retry(start_time, n_past_retries);
+                let total_retries = past_retries + middleware_retries;
+                let retry_decision = retry_policy.should_retry(start_time, total_retries);
                 if let reqwest_retry::RetryDecision::Retry { execute_after } = retry_decision {
                     debug!(
                         "Transient failure while handling response from {}; retrying...",
@@ -645,10 +723,15 @@ impl CachedClient {
                         .duration_since(SystemTime::now())
                         .unwrap_or_else(|_| Duration::default());
                     tokio::time::sleep(duration).await;
-                    n_past_retries += 1;
+                    past_retries += 1;
                     continue;
                 }
             }
+
+            if past_retries > 0 {
+                return result.map_err(|err| err.with_retries(past_retries));
+            }
+
             return result;
         }
     }
