@@ -152,9 +152,6 @@ pub enum ErrorKind {
     #[error(transparent)]
     InvalidUrl(#[from] uv_distribution_types::ToUrlError),
 
-    #[error(transparent)]
-    JoinRelativeUrl(#[from] uv_pypi_types::JoinRelativeError),
-
     #[error(transparent)]
     Flat(#[from] FlatIndexError),
 
@@ -197,6 +194,13 @@ pub enum ErrorKind {
     #[error("Failed to fetch: `{0}`")]
     WrappedReqwestError(DisplaySafeUrl, #[source] WrappedReqwestError),
 
+    /// Add the number of failed retries to the error.
+    #[error("Request failed after {retries} retries")]
+    RequestWithRetries {
+        source: Box<ErrorKind>,
+        retries: u32,
+    },
+
     #[error("Received some unexpected JSON from {}", url)]
     BadJson {
         source: serde_json::Error,
@@ -255,6 +259,9 @@ pub enum ErrorKind {
         "Network connectivity is disabled, but the requested data wasn't found in the cache for: `{0}`"
     )]
     Offline(String),
+
+    #[error("Invalid cache control header: `{0}`")]
+    InvalidCacheControl(String),
 }
 
 impl ErrorKind {
