@@ -3,14 +3,26 @@ use std::str::FromStr;
 
 use serde::{Deserialize, Serialize};
 use thiserror::Error;
-use url::Url;
 
 use uv_auth::{AuthPolicy, Credentials};
+use uv_redacted::DisplaySafeUrl;
+use uv_small_str::SmallString;
 
 use crate::index_name::{IndexName, IndexNameError};
 use crate::origin::Origin;
 use crate::{IndexStatusCodeStrategy, IndexUrl, IndexUrlError, SerializableStatusCode};
 
+/// Cache control configuration for an index.
+#[derive(Debug, Clone, Hash, Eq, PartialEq, Ord, PartialOrd, Serialize, Deserialize, Default)]
+#[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
+#[serde(rename_all = "kebab-case")]
+pub struct IndexCacheControl {
+    /// Cache control header for Simple API requests.
+    pub api: Option<SmallString>,
+    /// Cache control header for file downloads.
+    pub files: Option<SmallString>,
+}
+
 #[derive(Debug, Clone, Hash, Eq, PartialEq, Ord, PartialOrd, Serialize, Deserialize)]
 #[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
 #[serde(rename_all = "kebab-case")]
@@ -82,7 +94,7 @@ pub struct Index {
     /// url = "https://pypi.org/simple"
     /// publish-url = "https://upload.pypi.org/legacy/"
     /// ```
-    pub publish_url: Option<Url>,
+    pub publish_url: Option<DisplaySafeUrl>,
     /// When uv should use authentication for requests to the index.
     ///
     /// ```toml
@@ -104,6 +116,19 @@ pub struct Index {
     /// ```
     #[serde(default)]
     pub ignore_error_codes: Option<Vec<SerializableStatusCode>>,
+    /// Cache control configuration for this index.
+    ///
+    /// When set, these headers will override the server's cache control headers
+    /// for both package metadata requests and artifact downloads.
+    ///
+    /// ```toml
+    /// [[tool.uv.index]]
+    /// name = "my-index"
+    /// url = "https://<omitted>/simple"
+    /// cache-control = { api = "max-age=600", files = "max-age=3600" }
+    /// ```
+    #[serde(default)]
+    pub cache_control: Option<IndexCacheControl>,
 }
 
 #[derive(
@@ -142,6 +167,7 @@ impl Index {
             publish_url: None,
             authenticate: AuthPolicy::default(),
             ignore_error_codes: None,
+            cache_control: None,
         }
     }
 
@@ -157,6 +183,7 @@ impl Index {
             publish_url: None,
             authenticate: AuthPolicy::default(),
             ignore_error_codes: None,
+            cache_control: None,
         }
     }
 
@@ -172,6 +199,7 @@ impl Index {
             publish_url: None,
             authenticate: AuthPolicy::default(),
             ignore_error_codes: None,
+            cache_control: None,
         }
     }
 
@@ -193,7 +221,7 @@ impl Index {
     }
 
     /// Return the raw [`Url`] of the index.
-    pub fn raw_url(&self) -> &Url {
+    pub fn raw_url(&self) -> &DisplaySafeUrl {
         self.url.url()
     }
 
@@ -201,7 +229,7 @@ impl Index {
     ///
     /// For indexes with a `/simple` endpoint, this is simply the URL with the final segment
     /// removed. This is useful, e.g., for credential propagation to other endpoints on the index.
-    pub fn root_url(&self) -> Option<Url> {
+    pub fn root_url(&self) -> Option<DisplaySafeUrl> {
         self.url.root()
     }
 
@@ -250,6 +278,7 @@ impl From<IndexUrl> for Index {
             publish_url: None,
             authenticate: AuthPolicy::default(),
             ignore_error_codes: None,
+            cache_control: None,
         }
     }
 }
@@ -273,6 +302,7 @@ impl FromStr for Index {
                     publish_url: None,
                     authenticate: AuthPolicy::default(),
                     ignore_error_codes: None,
+                    cache_control: None,
                 });
             }
         }
@@ -289,6 +319,7 @@ impl FromStr for Index {
             publish_url: None,
             authenticate: AuthPolicy::default(),
             ignore_error_codes: None,
+            cache_control: None,
         })
     }
 }
@@ -384,3 +415,55 @@ pub enum IndexSourceError {
     #[error("Index included a name, but the name was empty")]
     EmptyName,
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_index_cache_control_headers() {
+        // Test that cache control headers are properly parsed from TOML
+        let toml_str = r#"
+            name = "test-index"
+            url = "https://test.example.com/simple"
+            cache-control = { api = "max-age=600", files = "max-age=3600" }
+        "#;
+
+        let index: Index = toml::from_str(toml_str).unwrap();
+        assert_eq!(index.name.as_ref().unwrap().as_ref(), "test-index");
+        assert!(index.cache_control.is_some());
+        let cache_control = index.cache_control.as_ref().unwrap();
+        assert_eq!(cache_control.api.as_deref(), Some("max-age=600"));
+        assert_eq!(cache_control.files.as_deref(), Some("max-age=3600"));
+    }
+
+    #[test]
+    fn test_index_without_cache_control() {
+        // Test that indexes work without cache control headers
+        let toml_str = r#"
+            name = "test-index"
+            url = "https://test.example.com/simple"
+        "#;
+
+        let index: Index = toml::from_str(toml_str).unwrap();
+        assert_eq!(index.name.as_ref().unwrap().as_ref(), "test-index");
+        assert_eq!(index.cache_control, None);
+    }
+
+    #[test]
+    fn test_index_partial_cache_control() {
+        // Test that cache control can have just one field
+        let toml_str = r#"
+            name = "test-index"
+            url = "https://test.example.com/simple"
+            cache-control = { api = "max-age=300" }
+        "#;
+
+        let index: Index = toml::from_str(toml_str).unwrap();
+        assert_eq!(index.name.as_ref().unwrap().as_ref(), "test-index");
+        assert!(index.cache_control.is_some());
+        let cache_control = index.cache_control.as_ref().unwrap();
+        assert_eq!(cache_control.api.as_deref(), Some("max-age=300"));
+        assert_eq!(cache_control.files, None);
+    }
+}
