@@ -599,6 +599,26 @@ impl<'a> IndexUrls {
         }
         IndexStatusCodeStrategy::Default
     }
+
+    /// Return the Simple API cache control header for an [`IndexUrl`], if configured.
+    pub fn simple_api_cache_control_for(&self, url: &IndexUrl) -> Option<&str> {
+        for index in &self.indexes {
+            if index.url() == url {
+                return index.cache_control.as_ref()?.api.as_deref();
+            }
+        }
+        None
+    }
+
+    /// Return the artifact cache control header for an [`IndexUrl`], if configured.
+    pub fn artifact_cache_control_for(&self, url: &IndexUrl) -> Option<&str> {
+        for index in &self.indexes {
+            if index.url() == url {
+                return index.cache_control.as_ref()?.files.as_deref();
+            }
+        }
+        None
+    }
 }
 
 bitflags::bitflags! {
@@ -717,4 +737,64 @@ mod tests {
             "git+https://github.com/example/repo.git"
         ));
     }
+
+    #[test]
+    fn test_cache_control_lookup() {
+        use std::str::FromStr;
+
+        use uv_small_str::SmallString;
+
+        use crate::IndexFormat;
+        use crate::index_name::IndexName;
+
+        let indexes = vec![
+            Index {
+                name: Some(IndexName::from_str("index1").unwrap()),
+                url: IndexUrl::from_str("https://index1.example.com/simple").unwrap(),
+                cache_control: Some(crate::IndexCacheControl {
+                    api: Some(SmallString::from("max-age=300")),
+                    files: Some(SmallString::from("max-age=1800")),
+                }),
+                explicit: false,
+                default: false,
+                origin: None,
+                format: IndexFormat::Simple,
+                publish_url: None,
+                authenticate: uv_auth::AuthPolicy::default(),
+                ignore_error_codes: None,
+            },
+            Index {
+                name: Some(IndexName::from_str("index2").unwrap()),
+                url: IndexUrl::from_str("https://index2.example.com/simple").unwrap(),
+                cache_control: None,
+                explicit: false,
+                default: false,
+                origin: None,
+                format: IndexFormat::Simple,
+                publish_url: None,
+                authenticate: uv_auth::AuthPolicy::default(),
+                ignore_error_codes: None,
+            },
+        ];
+
+        let index_urls = IndexUrls::from_indexes(indexes);
+
+        let url1 = IndexUrl::from_str("https://index1.example.com/simple").unwrap();
+        assert_eq!(
+            index_urls.simple_api_cache_control_for(&url1),
+            Some("max-age=300")
+        );
+        assert_eq!(
+            index_urls.artifact_cache_control_for(&url1),
+            Some("max-age=1800")
+        );
+
+        let url2 = IndexUrl::from_str("https://index2.example.com/simple").unwrap();
+        assert_eq!(index_urls.simple_api_cache_control_for(&url2), None);
+        assert_eq!(index_urls.artifact_cache_control_for(&url2), None);
+
+        let url3 = IndexUrl::from_str("https://index3.example.com/simple").unwrap();
+        assert_eq!(index_urls.simple_api_cache_control_for(&url3), None);
+        assert_eq!(index_urls.artifact_cache_control_for(&url3), None);
+    }
 }
