@@ -1,6 +1,5 @@
 use uv_distribution_filename::DistExtension;
 use uv_normalize::{ExtraName, GroupName, PackageName};
-use uv_pep508::MarkerTree;
 use uv_pypi_types::{HashDigest, HashDigests};
 
 use crate::{
@@ -146,7 +145,9 @@ impl Diagnostic for ResolutionDiagnostic {
                 format!("The package `{dist}` does not have an extra named `{extra}`")
             }
             Self::MissingDev { dist, dev } => {
-                format!("The package `{dist}` does not have a development dependency group named `{dev}`")
+                format!(
+                    "The package `{dist}` does not have a development dependency group named `{dev}`"
+                )
             }
             Self::YankedVersion { dist, reason } => {
                 if let Some(reason) = reason {
@@ -200,12 +201,12 @@ impl Node {
     }
 }
 
-/// An edge in the resolution graph, along with the marker that must be satisfied to traverse it.
+/// An edge in the resolution graph.
 #[derive(Debug, Clone)]
 pub enum Edge {
-    Prod(MarkerTree),
-    Optional(ExtraName, MarkerTree),
-    Dev(GroupName, MarkerTree),
+    Prod,
+    Optional(ExtraName),
+    Dev(GroupName),
 }
 
 impl From<&ResolvedDist> for RequirementSource {
