@@ -7,6 +7,7 @@ use owo_colors::OwoColorize;
 use tracing::debug;
 
 use uv_cache::Cache;
+use uv_configuration::PreviewMode;
 use uv_fs::{LockedFile, Simplified};
 use uv_pep440::Version;
 
@@ -152,13 +153,14 @@ impl PythonEnvironment {
         request: &PythonRequest,
         preference: EnvironmentPreference,
         cache: &Cache,
+        preview: PreviewMode,
     ) -> Result<Self, Error> {
         let installation = match find_python_installation(
             request,
             preference,
-            // Ignore managed installations when looking for environments
-            PythonPreference::OnlySystem,
+            PythonPreference::default(),
             cache,
+            preview,
         )? {
             Ok(installation) => installation,
             Err(err) => return Err(EnvironmentNotFound::from(err).into()),
@@ -171,7 +173,7 @@ impl PythonEnvironment {
     /// N.B. This function also works for system Python environments and users depend on this.
     pub fn from_root(root: impl AsRef<Path>, cache: &Cache) -> Result<Self, Error> {
         debug!(
-            "Checking for Python environment at `{}`",
+            "Checking for Python environment at: `{}`",
             root.as_ref().user_display()
         );
         match root.as_ref().try_exists() {
