@@ -1,4 +1,5 @@
 //! Find requested Python interpreters and query interpreters for information.
+use owo_colors::OwoColorize;
 use thiserror::Error;
 
 #[cfg(test)]
@@ -7,12 +8,17 @@ use uv_static::EnvVars;
 pub use crate::discovery::{
     EnvironmentPreference, Error as DiscoveryError, PythonDownloads, PythonNotFound,
     PythonPreference, PythonRequest, PythonSource, PythonVariant, VersionRequest,
-    find_python_installations,
+    find_python_installations, satisfies_python_preference,
 };
+pub use crate::downloads::PlatformRequest;
 pub use crate::environment::{InvalidEnvironmentKind, PythonEnvironment};
-pub use crate::implementation::ImplementationName;
-pub use crate::installation::{PythonInstallation, PythonInstallationKey};
-pub use crate::interpreter::{BrokenSymlink, Error as InterpreterError, Interpreter};
+pub use crate::implementation::{ImplementationName, LenientImplementationName};
+pub use crate::installation::{
+    PythonInstallation, PythonInstallationKey, PythonInstallationMinorVersionKey,
+};
+pub use crate::interpreter::{
+    BrokenSymlink, Error as InterpreterError, Interpreter, canonicalize_executable,
+};
 pub use crate::pointer_size::PointerSize;
 pub use crate::prefix::Prefix;
 pub use crate::python_version::PythonVersion;
@@ -88,8 +94,8 @@ pub enum Error {
     #[error(transparent)]
     KeyError(#[from] installation::PythonInstallationKeyError),
 
-    #[error(transparent)]
-    MissingPython(#[from] PythonNotFound),
+    #[error("{}{}", .0, if let Some(hint) = .1 { format!("\n\n{}{} {hint}", "hint".bold().cyan(), ":".bold()) } else { String::new() })]
+    MissingPython(PythonNotFound, Option<String>),
 
     #[error(transparent)]
     MissingEnvironment(#[from] environment::EnvironmentNotFound),
@@ -98,6 +104,21 @@ pub enum Error {
     InvalidEnvironment(#[from] environment::InvalidEnvironment),
 }
 
+impl Error {
+    pub(crate) fn with_missing_python_hint(self, hint: String) -> Self {
+        match self {
+            Error::MissingPython(err, _) => Error::MissingPython(err, Some(hint)),
+            _ => self,
+        }
+    }
+}
+
+impl From<PythonNotFound> for Error {
+    fn from(err: PythonNotFound) -> Self {
+        Error::MissingPython(err, None)
+    }
+}
+
 // The mock interpreters are not valid on Windows so we don't have unit test coverage there
 // TODO(zanieb): We should write a mock interpreter script that works on Windows
 #[cfg(all(test, unix))]
@@ -114,6 +135,7 @@ mod tests {
     use indoc::{formatdoc, indoc};
     use temp_env::with_vars;
     use test_log::test;
+    use uv_configuration::PreviewMode;
     use uv_static::EnvVars;
 
     use uv_cache::Cache;
@@ -446,6 +468,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
+                PreviewMode::Disabled,
             )
         });
         assert!(
@@ -460,6 +483,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
+                PreviewMode::Disabled,
             )
         });
         assert!(
@@ -484,6 +508,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
+                PreviewMode::Disabled,
             )
         });
         assert!(
@@ -505,6 +530,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert!(
@@ -566,6 +592,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert!(
@@ -597,6 +624,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
+                PreviewMode::Disabled,
             )
         });
         assert!(
@@ -633,6 +661,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::default(),
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert!(
@@ -664,6 +693,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -685,6 +715,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -710,6 +741,7 @@ mod tests {
                 EnvironmentPreference::OnlySystem,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -735,6 +767,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -757,6 +790,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
 
@@ -790,6 +824,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
 
@@ -823,6 +858,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })?;
         assert!(
@@ -844,6 +880,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })?;
         assert!(
@@ -865,6 +902,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
 
@@ -898,6 +936,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
 
@@ -934,6 +973,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })??;
         assert!(
@@ -964,6 +1004,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })??;
         assert!(
@@ -998,6 +1039,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })??;
         assert_eq!(
@@ -1023,6 +1065,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })??;
         assert_eq!(
@@ -1049,6 +1092,7 @@ mod tests {
                     EnvironmentPreference::OnlyVirtual,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )??;
@@ -1073,6 +1117,7 @@ mod tests {
                     EnvironmentPreference::OnlyVirtual,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )?;
@@ -1094,6 +1139,7 @@ mod tests {
                     EnvironmentPreference::OnlySystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )??;
@@ -1116,6 +1162,7 @@ mod tests {
                     EnvironmentPreference::OnlyVirtual,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )??;
@@ -1148,6 +1195,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )??;
@@ -1168,6 +1216,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )??;
@@ -1194,6 +1243,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
 
@@ -1211,6 +1261,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
 
@@ -1239,6 +1290,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })??;
         assert_eq!(
@@ -1276,6 +1328,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )??;
@@ -1303,6 +1356,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )??;
@@ -1327,6 +1381,7 @@ mod tests {
                     EnvironmentPreference::ExplicitSystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )??;
@@ -1351,6 +1406,7 @@ mod tests {
                     EnvironmentPreference::OnlySystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )??;
@@ -1375,6 +1431,7 @@ mod tests {
                     EnvironmentPreference::OnlyVirtual,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )??;
@@ -1412,6 +1469,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )??;
@@ -1439,6 +1497,7 @@ mod tests {
                     EnvironmentPreference::OnlySystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })??;
         assert_eq!(
@@ -1455,6 +1514,7 @@ mod tests {
                     EnvironmentPreference::OnlySystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })??;
         assert_eq!(
@@ -1471,6 +1531,7 @@ mod tests {
                     EnvironmentPreference::OnlySystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })?;
         assert!(
@@ -1492,6 +1553,7 @@ mod tests {
                 EnvironmentPreference::OnlyVirtual,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })?;
         assert!(
@@ -1508,6 +1570,7 @@ mod tests {
                     EnvironmentPreference::OnlySystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )?;
@@ -1529,6 +1592,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1543,6 +1607,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })?;
         assert!(
@@ -1556,6 +1621,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })?;
         assert!(
@@ -1584,6 +1650,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1599,6 +1666,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1628,6 +1696,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1643,6 +1712,7 @@ mod tests {
                 EnvironmentPreference::ExplicitSystem,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1658,6 +1728,7 @@ mod tests {
                 EnvironmentPreference::OnlyVirtual,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1673,6 +1744,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1696,6 +1768,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1710,6 +1783,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1733,6 +1807,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1752,6 +1827,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             },
         )??;
@@ -1780,6 +1856,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1801,6 +1878,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })?;
         assert!(
@@ -1830,6 +1908,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1845,6 +1924,7 @@ mod tests {
                 EnvironmentPreference::ExplicitSystem,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })?;
         assert!(
@@ -1871,6 +1951,7 @@ mod tests {
                     EnvironmentPreference::ExplicitSystem,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })
             .unwrap()
@@ -1895,6 +1976,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })?;
         assert!(
@@ -1911,6 +1993,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1925,6 +2008,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1950,6 +2034,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1964,6 +2049,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -1989,6 +2075,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -2015,6 +2102,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -2041,6 +2129,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -2067,6 +2156,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -2093,6 +2183,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -2120,6 +2211,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })?;
         assert!(
@@ -2141,6 +2233,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -2155,6 +2248,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -2180,6 +2274,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -2194,6 +2289,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
         assert_eq!(
@@ -2231,6 +2327,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })
             .unwrap()
@@ -2248,6 +2345,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })
             .unwrap()
@@ -2289,6 +2387,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })
             .unwrap()
@@ -2306,6 +2405,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })
             .unwrap()
@@ -2342,6 +2442,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })
             .unwrap()
@@ -2364,6 +2465,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })
             .unwrap()
@@ -2386,6 +2488,7 @@ mod tests {
                     EnvironmentPreference::Any,
                     PythonPreference::OnlySystem,
                     &context.cache,
+                    PreviewMode::Disabled,
                 )
             })
             .unwrap()
@@ -2424,6 +2527,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
 
@@ -2476,6 +2580,7 @@ mod tests {
                 EnvironmentPreference::Any,
                 PythonPreference::OnlySystem,
                 &context.cache,
+                PreviewMode::Disabled,
             )
         })??;
 
