@@ -23,12 +23,13 @@ pub(crate) use pip::tree::pip_tree;
 pub(crate) use pip::uninstall::pip_uninstall;
 pub(crate) use project::add::add;
 pub(crate) use project::export::export;
-pub(crate) use project::init::{init, InitKind, InitProjectKind};
+pub(crate) use project::init::{InitKind, InitProjectKind, init};
 pub(crate) use project::lock::lock;
 pub(crate) use project::remove::remove;
-pub(crate) use project::run::{run, RunCommand};
+pub(crate) use project::run::{RunCommand, run};
 pub(crate) use project::sync::sync;
 pub(crate) use project::tree::tree;
+pub(crate) use project::version::{project_version, self_version};
 pub(crate) use publish::publish;
 pub(crate) use python::dir::dir as python_dir;
 pub(crate) use python::find::find as python_find;
@@ -37,26 +38,26 @@ pub(crate) use python::install::install as python_install;
 pub(crate) use python::list::list as python_list;
 pub(crate) use python::pin::pin as python_pin;
 pub(crate) use python::uninstall::uninstall as python_uninstall;
+pub(crate) use python::update_shell::update_shell as python_update_shell;
 #[cfg(feature = "self-update")]
 pub(crate) use self_update::self_update;
 pub(crate) use tool::dir::dir as tool_dir;
 pub(crate) use tool::install::install as tool_install;
 pub(crate) use tool::list::list as tool_list;
-pub(crate) use tool::run::run as tool_run;
 pub(crate) use tool::run::ToolRunCommand;
+pub(crate) use tool::run::run as tool_run;
 pub(crate) use tool::uninstall::uninstall as tool_uninstall;
 pub(crate) use tool::update_shell::update_shell as tool_update_shell;
 pub(crate) use tool::upgrade::upgrade as tool_upgrade;
 use uv_cache::Cache;
 use uv_configuration::Concurrency;
 use uv_distribution_types::InstalledMetadata;
-use uv_fs::{Simplified, CWD};
+use uv_fs::{CWD, Simplified};
 use uv_installer::compile_tree;
 use uv_normalize::PackageName;
 use uv_python::PythonEnvironment;
 use uv_scripts::Pep723Script;
 pub(crate) use venv::venv;
-pub(crate) use version::{project_version, self_version};
 
 use crate::printer::Printer;
 
@@ -72,12 +73,10 @@ mod project;
 mod publish;
 mod python;
 pub(crate) mod reporters;
-mod run;
 #[cfg(feature = "self-update")]
 mod self_update;
 mod tool;
 mod venv;
-mod version;
 
 #[derive(Copy, Clone)]
 pub(crate) enum ExitStatus {
@@ -295,6 +294,7 @@ pub(super) fn capitalize(s: &str) -> String {
 
 /// A Python file that may or may not include an existing PEP 723 script tag.
 #[derive(Debug)]
+#[allow(clippy::large_enum_variant)]
 pub(crate) enum ScriptPath {
     /// The Python file already includes a PEP 723 script tag.
     Script(Pep723Script),
