@@ -2,13 +2,6 @@ use std::path::Path;
 
 use tracing::debug;
 
-use uv_cache::{Cache, CacheBucket};
-use uv_cache_key::{cache_digest, hash_digest};
-use uv_configuration::{Concurrency, Constraints, PreviewMode};
-use uv_distribution_types::{Name, Resolution};
-use uv_fs::PythonExt;
-use uv_python::{Interpreter, PythonEnvironment, canonicalize_executable};
-
 use crate::commands::pip::loggers::{InstallLogger, ResolveLogger};
 use crate::commands::pip::operations::Modifications;
 use crate::commands::project::{
@@ -17,6 +10,76 @@ use crate::commands::project::{
 use crate::printer::Printer;
 use crate::settings::{NetworkSettings, ResolverInstallerSettings};
 
+use uv_cache::{Cache, CacheBucket};
+use uv_cache_key::{cache_digest, hash_digest};
+use uv_configuration::{Concurrency, Constraints, PreviewMode};
+use uv_distribution_types::{Name, Resolution};
+use uv_fs::PythonExt;
+use uv_python::{Interpreter, PythonEnvironment, canonicalize_executable};
+
+/// An ephemeral [`PythonEnvironment`] for running an individual command.
+#[derive(Debug)]
+pub(crate) struct EphemeralEnvironment(PythonEnvironment);
+
+impl From<PythonEnvironment> for EphemeralEnvironment {
+    fn from(environment: PythonEnvironment) -> Self {
+        Self(environment)
+    }
+}
+
+impl From<EphemeralEnvironment> for PythonEnvironment {
+    fn from(environment: EphemeralEnvironment) -> Self {
+        environment.0
+    }
+}
+
+impl EphemeralEnvironment {
+    /// Set the ephemeral overlay for a Python environment.
+    #[allow(clippy::result_large_err)]
+    pub(crate) fn set_overlay(&self, contents: impl AsRef<[u8]>) -> Result<(), ProjectError> {
+        let site_packages = self
+            .0
+            .site_packages()
+            .next()
+            .ok_or(ProjectError::NoSitePackages)?;
+        let overlay_path = site_packages.join("_uv_ephemeral_overlay.pth");
+        fs_err::write(overlay_path, contents)?;
+        Ok(())
+    }
+
+    /// Enable system site packages for a Python environment.
+    #[allow(clippy::result_large_err)]
+    pub(crate) fn set_system_site_packages(&self) -> Result<(), ProjectError> {
+        self.0
+            .set_pyvenv_cfg("include-system-site-packages", "true")?;
+        Ok(())
+    }
+
+    /// Set the `extends-environment` key in the `pyvenv.cfg` file to the given path.
+    ///
+    /// Ephemeral environments created by `uv run --with` extend a parent (virtual or system)
+    /// environment by adding a `.pth` file to the ephemeral environment's `site-packages`
+    /// directory. The `pth` file contains Python code to dynamically add the parent
+    /// environment's `site-packages` directory to Python's import search paths in addition to
+    /// the ephemeral environment's `site-packages` directory. This works well at runtime, but
+    /// is too dynamic for static analysis tools like ty to understand. As such, we
+    /// additionally write the `sys.prefix` of the parent environment to to the
+    /// `extends-environment` key of the ephemeral environment's `pyvenv.cfg` file, making it
+    /// easier for these tools to statically and reliably understand the relationship between
+    /// the two environments.
+    #[allow(clippy::result_large_err)]
+    pub(crate) fn set_parent_environment(
+        &self,
+        parent_environment_sys_prefix: &Path,
+    ) -> Result<(), ProjectError> {
+        self.0.set_pyvenv_cfg(
+            "extends-environment",
+            &parent_environment_sys_prefix.escape_for_python(),
+        )?;
+        Ok(())
+    }
+}
+
 /// A [`PythonEnvironment`] stored in the cache.
 #[derive(Debug)]
 pub(crate) struct CachedEnvironment(PythonEnvironment);
@@ -44,15 +107,13 @@ impl CachedEnvironment {
         printer: Printer,
         preview: PreviewMode,
     ) -> Result<Self, ProjectError> {
-        // Resolve the "base" interpreter, which resolves to an underlying parent interpreter if the
-        // given interpreter is a virtual environment.
-        let base_interpreter = Self::base_interpreter(interpreter, cache)?;
+        let interpreter = Self::base_interpreter(interpreter, cache)?;
 
         // Resolve the requirements with the interpreter.
         let resolution = Resolution::from(
             resolve_environment(
                 spec,
-                &base_interpreter,
+                &interpreter,
                 build_constraints.clone(),
                 &settings.resolver,
                 network_settings,
@@ -80,29 +141,20 @@ impl CachedEnvironment {
         // Use the canonicalized base interpreter path since that's the interpreter we performed the
         // resolution with and the interpreter the environment will be created with.
         //
-        // We also include the canonicalized `sys.prefix` of the non-base interpreter, that is, the
-        // virtual environment's path. Originally, we shared cached environments independent of the
-        // environment they'd be layered on top of. However, this causes collisions as the overlay
-        // `.pth` file can be overridden by another instance of uv. Including this element in the key
-        // avoids this problem at the cost of creating separate cached environments for identical
-        // `--with` invocations across projects. We use `sys.prefix` rather than `sys.executable` so
-        // we can canonicalize it without invalidating the purpose of the element — it'd probably be
-        // safe to just use the absolute `sys.executable` as well.
-        //
-        // TODO(zanieb): Since we're not sharing these environmments across projects, we should move
-        // [`CachedEvnvironment::set_overlay`] etc. here since the values there should be constant
-        // now.
+        // We cache environments independent of the environment they'd be layered on top of. The
+        // assumption is such that the environment will _not_ be modified by the user or uv;
+        // otherwise, we risk cache poisoning. For example, if we were to write a `.pth` file to
+        // the cached environment, it would be shared across all projects that use the same
+        // interpreter and the same cached dependencies.
         //
         // TODO(zanieb): We should include the version of the base interpreter in the hash, so if
         // the interpreter at the canonicalized path changes versions we construct a new
         // environment.
-        let environment_hash = cache_digest(&(
-            &canonicalize_executable(base_interpreter.sys_executable())?,
-            &interpreter.sys_prefix().canonicalize()?,
-        ));
+        let interpreter_hash =
+            cache_digest(&canonicalize_executable(interpreter.sys_executable())?);
 
         // Search in the content-addressed cache.
-        let cache_entry = cache.entry(CacheBucket::Environments, environment_hash, resolution_hash);
+        let cache_entry = cache.entry(CacheBucket::Environments, interpreter_hash, resolution_hash);
 
         if cache.refresh().is_none() {
             if let Ok(root) = cache.resolve_link(cache_entry.path()) {
@@ -116,10 +168,10 @@ impl CachedEnvironment {
         let temp_dir = cache.venv_dir()?;
         let venv = uv_virtualenv::create_venv(
             temp_dir.path(),
-            base_interpreter,
+            interpreter,
             uv_virtualenv::Prompt::None,
             false,
-            false,
+            uv_virtualenv::OnExisting::Remove,
             true,
             false,
             false,
@@ -150,76 +202,6 @@ impl CachedEnvironment {
         Ok(Self(PythonEnvironment::from_root(root, cache)?))
     }
 
-    /// Set the ephemeral overlay for a Python environment.
-    #[allow(clippy::result_large_err)]
-    pub(crate) fn set_overlay(&self, contents: impl AsRef<[u8]>) -> Result<(), ProjectError> {
-        let site_packages = self
-            .0
-            .site_packages()
-            .next()
-            .ok_or(ProjectError::NoSitePackages)?;
-        let overlay_path = site_packages.join("_uv_ephemeral_overlay.pth");
-        fs_err::write(overlay_path, contents)?;
-        Ok(())
-    }
-
-    /// Clear the ephemeral overlay for a Python environment, if it exists.
-    #[allow(clippy::result_large_err)]
-    pub(crate) fn clear_overlay(&self) -> Result<(), ProjectError> {
-        let site_packages = self
-            .0
-            .site_packages()
-            .next()
-            .ok_or(ProjectError::NoSitePackages)?;
-        let overlay_path = site_packages.join("_uv_ephemeral_overlay.pth");
-        match fs_err::remove_file(overlay_path) {
-            Ok(()) => (),
-            Err(err) if err.kind() == std::io::ErrorKind::NotFound => (),
-            Err(err) => return Err(ProjectError::OverlayRemoval(err)),
-        }
-        Ok(())
-    }
-
-    /// Enable system site packages for a Python environment.
-    #[allow(clippy::result_large_err)]
-    pub(crate) fn set_system_site_packages(&self) -> Result<(), ProjectError> {
-        self.0
-            .set_pyvenv_cfg("include-system-site-packages", "true")?;
-        Ok(())
-    }
-
-    /// Disable system site packages for a Python environment.
-    #[allow(clippy::result_large_err)]
-    pub(crate) fn clear_system_site_packages(&self) -> Result<(), ProjectError> {
-        self.0
-            .set_pyvenv_cfg("include-system-site-packages", "false")?;
-        Ok(())
-    }
-
-    /// Set the `extends-environment` key in the `pyvenv.cfg` file to the given path.
-    ///
-    /// Ephemeral environments created by `uv run --with` extend a parent (virtual or system)
-    /// environment by adding a `.pth` file to the ephemeral environment's `site-packages`
-    /// directory. The `pth` file contains Python code to dynamically add the parent
-    /// environment's `site-packages` directory to Python's import search paths in addition to
-    /// the ephemeral environment's `site-packages` directory. This works well at runtime, but
-    /// is too dynamic for static analysis tools like ty to understand. As such, we
-    /// additionally write the `sys.prefix` of the parent environment to to the
-    /// `extends-environment` key of the ephemeral environment's `pyvenv.cfg` file, making it
-    /// easier for these tools to statically and reliably understand the relationship between
-    /// the two environments.
-    #[allow(clippy::result_large_err)]
-    pub(crate) fn set_parent_environment(
-        &self,
-        parent_environment_sys_prefix: &Path,
-    ) -> Result<(), ProjectError> {
-        self.0.set_pyvenv_cfg(
-            "extends-environment",
-            &parent_environment_sys_prefix.escape_for_python(),
-        )?;
-        Ok(())
-    }
-
     /// Return the [`Interpreter`] to use for the cached environment, based on a given
     /// [`Interpreter`].
     ///
