@@ -4,13 +4,15 @@ use std::fmt::Write;
 use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
 use std::str::FromStr;
+use uv_distribution_types::RequiresPython;
 
 use tracing::{debug, trace, warn};
 use uv_cache::Cache;
 use uv_cli::AuthorFrom;
 use uv_client::BaseClientBuilder;
 use uv_configuration::{
-    PreviewMode, ProjectBuildBackend, VersionControlError, VersionControlSystem,
+    DependencyGroupsWithDefaults, PreviewMode, ProjectBuildBackend, VersionControlError,
+    VersionControlSystem,
 };
 use uv_fs::{CWD, Simplified};
 use uv_git::GIT;
@@ -21,7 +23,6 @@ use uv_python::{
     PythonPreference, PythonRequest, PythonVariant, PythonVersionFile, VersionFileDiscoveryOptions,
     VersionRequest,
 };
-use uv_resolver::RequiresPython;
 use uv_scripts::{Pep723Script, ScriptTag};
 use uv_settings::PythonInstallMirrors;
 use uv_static::EnvVars;
@@ -62,9 +63,6 @@ pub(crate) async fn init(
     printer: Printer,
     preview: PreviewMode,
 ) -> Result<ExitStatus> {
-    if build_backend == Some(ProjectBuildBackend::Uv) && preview.is_disabled() {
-        warn_user_once!("The uv build backend is experimental and may change without warning");
-    }
     match init_kind {
         InitKind::Script => {
             let Some(path) = explicit_path.as_deref() else {
@@ -86,6 +84,7 @@ pub(crate) async fn init(
                 pin_python,
                 package,
                 no_config,
+                preview,
             )
             .await?;
 
@@ -201,6 +200,7 @@ async fn init_script(
     pin_python: bool,
     package: bool,
     no_config: bool,
+    preview: PreviewMode,
 ) -> Result<()> {
     if no_workspace {
         warn_user_once!("`--no-workspace` is a no-op for Python scripts, which are standalone");
@@ -215,6 +215,7 @@ async fn init_script(
         warn_user_once!("`--package` is a no-op for Python scripts, which are standalone");
     }
     let client_builder = BaseClientBuilder::new()
+        .retries_from_env()?
         .connectivity(network_settings.connectivity)
         .native_tls(network_settings.native_tls)
         .allow_insecure_host(network_settings.allow_insecure_host.clone());
@@ -257,6 +258,7 @@ async fn init_script(
         &client_builder,
         cache,
         &reporter,
+        preview,
     )
     .await?;
 
@@ -344,6 +346,7 @@ async fn init_project(
 
     let reporter = PythonDownloadReporter::single(printer);
     let client_builder = BaseClientBuilder::new()
+        .retries_from_env()?
         .connectivity(network_settings.connectivity)
         .native_tls(network_settings.native_tls)
         .allow_insecure_host(network_settings.allow_insecure_host.clone());
@@ -433,6 +436,7 @@ async fn init_project(
                         install_mirrors.python_install_mirror.as_deref(),
                         install_mirrors.pypy_install_mirror.as_deref(),
                         install_mirrors.python_downloads_json_url.as_deref(),
+                        preview,
                     )
                     .await?
                     .into_interpreter();
@@ -460,6 +464,7 @@ async fn init_project(
                     install_mirrors.python_install_mirror.as_deref(),
                     install_mirrors.pypy_install_mirror.as_deref(),
                     install_mirrors.python_downloads_json_url.as_deref(),
+                    preview,
                 )
                 .await?
                 .into_interpreter();
@@ -502,7 +507,7 @@ async fn init_project(
         (requires_python, python_request)
     } else if let Some(requires_python) = workspace
         .as_ref()
-        .map(find_requires_python)
+        .map(|workspace| find_requires_python(workspace, &DependencyGroupsWithDefaults::none()))
         .transpose()?
         .flatten()
     {
@@ -526,6 +531,7 @@ async fn init_project(
                 install_mirrors.python_install_mirror.as_deref(),
                 install_mirrors.pypy_install_mirror.as_deref(),
                 install_mirrors.python_downloads_json_url.as_deref(),
+                preview,
             )
             .await?
             .into_interpreter();
@@ -553,6 +559,7 @@ async fn init_project(
             install_mirrors.python_install_mirror.as_deref(),
             install_mirrors.pypy_install_mirror.as_deref(),
             install_mirrors.python_downloads_json_url.as_deref(),
+            preview,
         )
         .await?
         .into_interpreter();
@@ -586,7 +593,6 @@ async fn init_project(
         author_from,
         no_readme,
         package,
-        preview,
     )?;
 
     if let Some(workspace) = workspace {
@@ -714,7 +720,6 @@ impl InitProjectKind {
         author_from: Option<AuthorFrom>,
         no_readme: bool,
         package: bool,
-        preview: PreviewMode,
     ) -> Result<()> {
         match self {
             InitProjectKind::Application => InitProjectKind::init_application(
@@ -729,7 +734,6 @@ impl InitProjectKind {
                 author_from,
                 no_readme,
                 package,
-                preview,
             ),
             InitProjectKind::Library => InitProjectKind::init_library(
                 name,
@@ -743,7 +747,6 @@ impl InitProjectKind {
                 author_from,
                 no_readme,
                 package,
-                preview,
             ),
         }
     }
@@ -762,7 +765,6 @@ impl InitProjectKind {
         author_from: Option<AuthorFrom>,
         no_readme: bool,
         package: bool,
-        preview: PreviewMode,
     ) -> Result<()> {
         fs_err::create_dir_all(path)?;
 
@@ -795,11 +797,7 @@ impl InitProjectKind {
             }
 
             // Add a build system
-            let build_backend = match build_backend {
-                Some(build_backend) => build_backend,
-                None if preview.is_enabled() => ProjectBuildBackend::Uv,
-                None => ProjectBuildBackend::Hatch,
-            };
+            let build_backend = build_backend.unwrap_or(ProjectBuildBackend::Uv);
             pyproject.push('\n');
             pyproject.push_str(&pyproject_build_system(name, build_backend));
             pyproject_build_backend_prerequisites(name, path, build_backend)?;
@@ -849,7 +847,6 @@ impl InitProjectKind {
         author_from: Option<AuthorFrom>,
         no_readme: bool,
         package: bool,
-        preview: PreviewMode,
     ) -> Result<()> {
         if !package {
             return Err(anyhow!("Library projects must be packaged"));
@@ -870,11 +867,7 @@ impl InitProjectKind {
         );
 
         // Always include a build system if the project is packaged.
-        let build_backend = match build_backend {
-            Some(build_backend) => build_backend,
-            None if preview.is_enabled() => ProjectBuildBackend::Uv,
-            None => ProjectBuildBackend::Hatch,
-        };
+        let build_backend = build_backend.unwrap_or(ProjectBuildBackend::Uv);
         pyproject.push('\n');
         pyproject.push_str(&pyproject_build_system(name, build_backend));
         pyproject_build_backend_prerequisites(name, path, build_backend)?;
