@@ -135,6 +135,14 @@ impl Changelog {
     }
 }
 
+#[derive(Debug, Clone, Copy)]
+enum InstallErrorKind {
+    DownloadUnpack,
+    Bin,
+    #[cfg(windows)]
+    Registry,
+}
+
 /// Download and install Python versions.
 #[allow(clippy::fn_params_excessive_bools)]
 pub(crate) async fn install(
@@ -143,6 +151,8 @@ pub(crate) async fn install(
     targets: Vec<String>,
     reinstall: bool,
     upgrade: bool,
+    bin: Option<bool>,
+    registry: Option<bool>,
     force: bool,
     python_install_mirror: Option<String>,
     pypy_install_mirror: Option<String>,
@@ -156,12 +166,14 @@ pub(crate) async fn install(
 ) -> Result<ExitStatus> {
     let start = std::time::Instant::now();
 
+    // TODO(zanieb): We should consider marking the Python installation as the default when
+    // `--default` is used. It's not clear how this overlaps with a global Python pin, but I'd be
+    // surprised if `uv python find` returned the "newest" Python version rather than the one I just
+    // installed with the `--default` flag.
     if default && !preview.is_enabled() {
-        writeln!(
-            printer.stderr(),
-            "The `--default` flag is only available in preview mode; add the `--preview` flag to use `--default`"
-        )?;
-        return Ok(ExitStatus::Failure);
+        warn_user!(
+            "The `--default` option is experimental and may change without warning. Pass `--preview` to disable this warning"
+        );
     }
 
     if upgrade && preview.is_disabled() {
@@ -212,6 +224,8 @@ pub(crate) async fn install(
             .map(PythonVersionFile::into_versions)
             .unwrap_or_else(|| {
                 // If no version file is found and no requests were made
+                // TODO(zanieb): We should consider differentiating between a global Python version
+                // file here, allowing a request from there to enable `is_default_install`.
                 is_default_install = true;
                 vec![if reinstall {
                     // On bare `--reinstall`, reinstall all Python versions
@@ -376,6 +390,7 @@ pub(crate) async fn install(
 
     // Download and unpack the Python versions concurrently
     let client = uv_client::BaseClientBuilder::new()
+        .retries_from_env()?
         .connectivity(network_settings.connectivity)
         .native_tls(network_settings.native_tls)
         .allow_insecure_host(network_settings.allow_insecure_host.clone())
@@ -431,15 +446,19 @@ pub(crate) async fn install(
                 downloaded.push(installation.clone());
             }
             Err(err) => {
-                errors.push((download.key().clone(), anyhow::Error::new(err)));
+                errors.push((
+                    InstallErrorKind::DownloadUnpack,
+                    download.key().clone(),
+                    anyhow::Error::new(err),
+                ));
             }
         }
     }
 
-    let bin = if preview.is_enabled() {
-        Some(python_executable_dir()?)
-    } else {
+    let bin_dir = if matches!(bin, Some(false)) {
         None
+    } else {
+        Some(python_executable_dir()?)
     };
 
     let installations: Vec<_> = downloaded.iter().chain(satisfied.iter().copied()).collect();
@@ -454,40 +473,41 @@ pub(crate) async fn install(
             e.warn_user(installation);
         }
 
-        if preview.is_disabled() {
-            debug!("Skipping installation of Python executables, use `--preview` to enable.");
-            continue;
-        }
-
-        let bin = bin
-            .as_ref()
-            .expect("We should have a bin directory with preview enabled")
-            .as_path();
-
         let upgradeable = (default || is_default_install)
             || requested_minor_versions.contains(&installation.key().version().python_version());
 
-        create_bin_links(
-            installation,
-            bin,
-            reinstall,
-            force,
-            default,
-            upgradeable,
-            upgrade,
-            is_default_install,
-            first_request,
-            &existing_installations,
-            &installations,
-            &mut changelog,
-            &mut errors,
-            preview,
-        )?;
+        if let Some(bin_dir) = bin_dir.as_ref() {
+            create_bin_links(
+                installation,
+                bin_dir,
+                reinstall,
+                force,
+                default,
+                upgradeable,
+                upgrade,
+                is_default_install,
+                first_request,
+                &existing_installations,
+                &installations,
+                &mut changelog,
+                &mut errors,
+                preview,
+            );
+        }
 
-        if preview.is_enabled() {
+        if !matches!(registry, Some(false)) {
             #[cfg(windows)]
             {
-                uv_python::windows_registry::create_registry_entry(installation, &mut errors)?;
+                match uv_python::windows_registry::create_registry_entry(installation) {
+                    Ok(()) => {}
+                    Err(err) => {
+                        errors.push((
+                            InstallErrorKind::Registry,
+                            installation.key().clone(),
+                            err.into(),
+                        ));
+                    }
+                }
             }
         }
     }
@@ -635,24 +655,50 @@ pub(crate) async fn install(
             }
         }
 
-        if preview.is_enabled() {
-            let bin = bin
-                .as_ref()
-                .expect("We should have a bin directory with preview enabled")
-                .as_path();
-            warn_if_not_on_path(bin);
+        if let Some(bin_dir) = bin_dir.as_ref() {
+            warn_if_not_on_path(bin_dir);
         }
     }
 
     if !errors.is_empty() {
-        for (key, err) in errors
+        // If there are only side-effect install errors and the user didn't opt-in, we're only going
+        // to warn
+        let fatal = !errors.iter().all(|(kind, _, _)| match kind {
+            InstallErrorKind::Bin => bin.is_none(),
+            #[cfg(windows)]
+            InstallErrorKind::Registry => registry.is_none(),
+            InstallErrorKind::DownloadUnpack => false,
+        });
+
+        for (kind, key, err) in errors
             .into_iter()
-            .sorted_unstable_by(|(key_a, _), (key_b, _)| key_a.cmp(key_b))
+            .sorted_unstable_by(|(_, key_a, _), (_, key_b, _)| key_a.cmp(key_b))
         {
+            let (level, verb) = match kind {
+                InstallErrorKind::DownloadUnpack => ("error".red().bold().to_string(), "install"),
+                InstallErrorKind::Bin => {
+                    let level = match bin {
+                        None => "warning".yellow().bold().to_string(),
+                        Some(false) => continue,
+                        Some(true) => "error".red().bold().to_string(),
+                    };
+                    (level, "install executable for")
+                }
+                #[cfg(windows)]
+                InstallErrorKind::Registry => {
+                    let level = match registry {
+                        None => "warning".yellow().bold().to_string(),
+                        Some(false) => continue,
+                        Some(true) => "error".red().bold().to_string(),
+                    };
+                    (level, "install registry entry for")
+                }
+            };
+
             writeln!(
                 printer.stderr(),
-                "{}: Failed to install {}",
-                "error".red().bold(),
+                "{level}{} Failed to {verb} {}",
+                ":".bold(),
                 key.green()
             )?;
             for err in err.chain() {
@@ -664,13 +710,18 @@ pub(crate) async fn install(
                 )?;
             }
         }
-        return Ok(ExitStatus::Failure);
+
+        if fatal {
+            return Ok(ExitStatus::Failure);
+        }
     }
 
     Ok(ExitStatus::Success)
 }
 
 /// Link the binaries of a managed Python installation to the bin directory.
+///
+/// This function is fallible, but errors are pushed to `errors` instead of being thrown.
 #[allow(clippy::fn_params_excessive_bools)]
 fn create_bin_links(
     installation: &ManagedPythonInstallation,
@@ -685,19 +736,23 @@ fn create_bin_links(
     existing_installations: &[ManagedPythonInstallation],
     installations: &[&ManagedPythonInstallation],
     changelog: &mut Changelog,
-    errors: &mut Vec<(PythonInstallationKey, Error)>,
+    errors: &mut Vec<(InstallErrorKind, PythonInstallationKey, Error)>,
     preview: PreviewMode,
-) -> Result<(), Error> {
-    let targets =
-        if (default || is_default_install) && first_request.matches_installation(installation) {
-            vec![
-                installation.key().executable_name_minor(),
-                installation.key().executable_name_major(),
-                installation.key().executable_name(),
-            ]
-        } else {
-            vec![installation.key().executable_name_minor()]
-        };
+) {
+    // TODO(zanieb): We want more feedback on the `is_default_install` behavior before stabilizing
+    // it. In particular, it may be confusing because it does not apply when versions are loaded
+    // from a `.python-version` file.
+    let targets = if (default || (is_default_install && preview.is_enabled()))
+        && first_request.matches_installation(installation)
+    {
+        vec![
+            installation.key().executable_name_minor(),
+            installation.key().executable_name_major(),
+            installation.key().executable_name(),
+        ]
+    } else {
+        vec![installation.key().executable_name_minor()]
+    };
 
     for target in targets {
         let target = bin.join(target);
@@ -772,6 +827,7 @@ fn create_bin_links(
                                     );
                                 } else {
                                     errors.push((
+                                        InstallErrorKind::Bin,
                                         installation.key().clone(),
                                         anyhow::anyhow!(
                                             "Executable already exists at `{}` but is not managed by uv; use `--force` to replace it",
@@ -847,7 +903,17 @@ fn create_bin_links(
                 }
 
                 // Replace the existing link
-                fs_err::remove_file(&to)?;
+                if let Err(err) = fs_err::remove_file(&to) {
+                    errors.push((
+                        InstallErrorKind::Bin,
+                        installation.key().clone(),
+                        anyhow::anyhow!(
+                            "Executable already exists at `{}` but could not be removed: {err}",
+                            to.simplified_display()
+                        ),
+                    ));
+                    continue;
+                }
 
                 if let Some(existing) = existing {
                     // Ensure we do not report installation of this executable for an existing
@@ -859,7 +925,18 @@ fn create_bin_links(
                         .remove(&target);
                 }
 
-                create_link_to_executable(&target, executable)?;
+                if let Err(err) = create_link_to_executable(&target, executable) {
+                    errors.push((
+                        InstallErrorKind::Bin,
+                        installation.key().clone(),
+                        anyhow::anyhow!(
+                            "Failed to create link at `{}`: {err}",
+                            target.simplified_display()
+                        ),
+                    ));
+                    continue;
+                }
+
                 debug!(
                     "Updated executable at `{}` to {}",
                     target.simplified_display(),
@@ -873,11 +950,14 @@ fn create_bin_links(
                     .insert(target.clone());
             }
             Err(err) => {
-                errors.push((installation.key().clone(), anyhow::Error::new(err)));
+                errors.push((
+                    InstallErrorKind::Bin,
+                    installation.key().clone(),
+                    anyhow::Error::new(err),
+                ));
             }
         }
     }
-    Ok(())
 }
 
 pub(crate) fn format_executables(
@@ -907,20 +987,29 @@ fn warn_if_not_on_path(bin: &Path) {
     if !Shell::contains_path(bin) {
         if let Some(shell) = Shell::from_env() {
             if let Some(command) = shell.prepend_path(bin) {
-                warn_user!(
-                    "`{}` is not on your PATH. To use the installed Python executable, run `{}`.",
-                    bin.simplified_display().cyan(),
-                    command.green(),
-                );
+                if shell.supports_update() {
+                    warn_user!(
+                        "`{}` is not on your PATH. To use installed Python executables, run `{}` or `{}`.",
+                        bin.simplified_display().cyan(),
+                        command.green(),
+                        "uv python update-shell".green()
+                    );
+                } else {
+                    warn_user!(
+                        "`{}` is not on your PATH. To use installed Python executables, run `{}`.",
+                        bin.simplified_display().cyan(),
+                        command.green()
+                    );
+                }
             } else {
                 warn_user!(
-                    "`{}` is not on your PATH. To use the installed Python executable, add the directory to your PATH.",
+                    "`{}` is not on your PATH. To use installed Python executables, add the directory to your PATH.",
                     bin.simplified_display().cyan(),
                 );
             }
         } else {
             warn_user!(
-                "`{}` is not on your PATH. To use the installed Python executable, add the directory to your PATH.",
+                "`{}` is not on your PATH. To use installed Python executables, add the directory to your PATH.",
                 bin.simplified_display().cyan(),
             );
         }
