@@ -318,15 +318,15 @@ fn python_find_project() {
     "###);
 
     // Unless explicitly requested
-    uv_snapshot!(context.filters(), context.python_find().arg("3.10"), @r###"
+    uv_snapshot!(context.filters(), context.python_find().arg("3.10"), @r"
     success: true
     exit_code: 0
     ----- stdout -----
     [PYTHON-3.10]
 
     ----- stderr -----
-    warning: The requested interpreter resolved to Python 3.10.[X], which is incompatible with the project's Python requirement: `>=3.11`
-    "###);
+    warning: The requested interpreter resolved to Python 3.10.[X], which is incompatible with the project's Python requirement: `>=3.11` (from `project.requires-python`)
+    ");
 
     // Or `--no-project` is used
     uv_snapshot!(context.filters(), context.python_find().arg("--no-project"), @r###"
@@ -367,15 +367,16 @@ fn python_find_project() {
     "###);
 
     // We should warn on subsequent uses, but respect the pinned version?
-    uv_snapshot!(context.filters(), context.python_find(), @r###"
+    uv_snapshot!(context.filters(), context.python_find(), @r"
     success: true
     exit_code: 0
     ----- stdout -----
     [PYTHON-3.10]
 
     ----- stderr -----
-    warning: The Python request from `.python-version` resolved to Python 3.10.[X], which is incompatible with the project's Python requirement: `>=3.11`. Use `uv python pin` to update the `.python-version` file to a compatible version.
-    "###);
+    warning: The Python request from `.python-version` resolved to Python 3.10.[X], which is incompatible with the project's Python requirement: `>=3.11` (from `project.requires-python`)
+    Use `uv python pin` to update the `.python-version` file to a compatible version
+    ");
 
     // Unless the pin file is outside the project, in which case we should just ignore it
     let child_dir = context.temp_dir.child("child");
@@ -424,25 +425,25 @@ fn python_find_venv() {
     // is super annoying and requires some changes to how we represent working directories in the
     // test context to resolve.
     #[cfg(not(windows))]
-    uv_snapshot!(context.filters(), context.python_find(), @r###"
+    uv_snapshot!(context.filters(), context.python_find(), @r"
     success: true
     exit_code: 0
     ----- stdout -----
-    [VENV]/[BIN]/python
+    [VENV]/[BIN]/[PYTHON]
 
     ----- stderr -----
-    "###);
+    ");
 
     // Even if the `VIRTUAL_ENV` is not set (the test context includes this by default)
     #[cfg(not(windows))]
-    uv_snapshot!(context.filters(), context.python_find().env_remove(EnvVars::VIRTUAL_ENV), @r###"
+    uv_snapshot!(context.filters(), context.python_find().env_remove(EnvVars::VIRTUAL_ENV), @r"
     success: true
     exit_code: 0
     ----- stdout -----
-    [VENV]/[BIN]/python
+    [VENV]/[BIN]/[PYTHON]
 
     ----- stderr -----
-    "###);
+    ");
 
     let child_dir = context.temp_dir.child("child");
     child_dir.create_dir_all().unwrap();
@@ -484,14 +485,14 @@ fn python_find_venv() {
 
     // We should find virtual environments from a child directory
     #[cfg(not(windows))]
-    uv_snapshot!(context.filters(), context.python_find().current_dir(&child_dir).env_remove(EnvVars::VIRTUAL_ENV), @r###"
+    uv_snapshot!(context.filters(), context.python_find().current_dir(&child_dir).env_remove(EnvVars::VIRTUAL_ENV), @r"
     success: true
     exit_code: 0
     ----- stdout -----
-    [VENV]/[BIN]/python
+    [VENV]/[BIN]/[PYTHON]
 
     ----- stderr -----
-    "###);
+    ");
 
     // A virtual environment in the child directory takes precedence over the parent
     uv_snapshot!(context.filters(), context.venv().arg("--python").arg("3.11").arg("-q").current_dir(&child_dir), @r###"
@@ -503,14 +504,14 @@ fn python_find_venv() {
     "###);
 
     #[cfg(not(windows))]
-    uv_snapshot!(context.filters(), context.python_find().current_dir(&child_dir).env_remove(EnvVars::VIRTUAL_ENV), @r###"
+    uv_snapshot!(context.filters(), context.python_find().current_dir(&child_dir).env_remove(EnvVars::VIRTUAL_ENV), @r"
     success: true
     exit_code: 0
     ----- stdout -----
-    [TEMP_DIR]/child/.venv/[BIN]/python
+    [TEMP_DIR]/child/.venv/[BIN]/[PYTHON]
 
     ----- stderr -----
-    "###);
+    ");
 
     // But if we delete the parent virtual environment
     fs_err::remove_dir_all(context.temp_dir.child(".venv")).unwrap();
@@ -527,36 +528,36 @@ fn python_find_venv() {
 
     // Unless, it is requested by path
     #[cfg(not(windows))]
-    uv_snapshot!(context.filters(), context.python_find().arg("child/.venv"), @r###"
+    uv_snapshot!(context.filters(), context.python_find().arg("child/.venv"), @r"
     success: true
     exit_code: 0
     ----- stdout -----
-    [TEMP_DIR]/child/.venv/[BIN]/python
+    [TEMP_DIR]/child/.venv/[BIN]/[PYTHON]
 
     ----- stderr -----
-    "###);
+    ");
 
     // Or activated via `VIRTUAL_ENV`
     #[cfg(not(windows))]
-    uv_snapshot!(context.filters(), context.python_find().env(EnvVars::VIRTUAL_ENV, child_dir.join(".venv").as_os_str()), @r###"
+    uv_snapshot!(context.filters(), context.python_find().env(EnvVars::VIRTUAL_ENV, child_dir.join(".venv").as_os_str()), @r"
     success: true
     exit_code: 0
     ----- stdout -----
-    [TEMP_DIR]/child/.venv/[BIN]/python
+    [TEMP_DIR]/child/.venv/[BIN]/[PYTHON]
 
     ----- stderr -----
-    "###);
+    ");
 
     // Or at the front of the PATH
     #[cfg(not(windows))]
-    uv_snapshot!(context.filters(), context.python_find().env(EnvVars::UV_TEST_PYTHON_PATH, child_dir.join(".venv").join("bin").as_os_str()), @r###"
+    uv_snapshot!(context.filters(), context.python_find().env(EnvVars::UV_TEST_PYTHON_PATH, child_dir.join(".venv").join("bin").as_os_str()), @r"
     success: true
     exit_code: 0
     ----- stdout -----
-    [TEMP_DIR]/child/.venv/[BIN]/python
+    [TEMP_DIR]/child/.venv/[BIN]/[PYTHON]
 
     ----- stderr -----
-    "###);
+    ");
 
     // This holds even if there are other directories before it in the path, as long as they do
     // not contain a Python executable
@@ -568,14 +569,14 @@ fn python_find_venv() {
         ])
         .unwrap();
 
-        uv_snapshot!(context.filters(), context.python_find().env(EnvVars::UV_TEST_PYTHON_PATH, path.as_os_str()), @r###"
+        uv_snapshot!(context.filters(), context.python_find().env(EnvVars::UV_TEST_PYTHON_PATH, path.as_os_str()), @r"
         success: true
         exit_code: 0
         ----- stdout -----
-        [TEMP_DIR]/child/.venv/[BIN]/python
+        [TEMP_DIR]/child/.venv/[BIN]/[PYTHON]
 
         ----- stderr -----
-        "###);
+        ");
     }
 
     // But, if there's an executable _before_ the virtual environment — we prefer that
@@ -677,33 +678,32 @@ fn python_find_unsupported_version() {
 #[test]
 fn python_find_venv_invalid() {
     let context: TestContext = TestContext::new("3.12")
-        // Enable additional filters for Windows compatibility
-        .with_filtered_exe_suffix()
         .with_filtered_python_names()
-        .with_filtered_virtualenv_bin();
+        .with_filtered_virtualenv_bin()
+        .with_filtered_exe_suffix();
 
     // We find the virtual environment
-    uv_snapshot!(context.filters(), context.python_find().env(EnvVars::VIRTUAL_ENV, context.venv.as_os_str()), @r###"
+    uv_snapshot!(context.filters(), context.python_find().env(EnvVars::VIRTUAL_ENV, context.venv.as_os_str()), @r"
     success: true
     exit_code: 0
     ----- stdout -----
-    [VENV]/[BIN]/python
+    [VENV]/[BIN]/[PYTHON]
 
     ----- stderr -----
-    "###);
+    ");
 
     // If the binaries are missing from a virtual environment, we fail
     fs_err::remove_dir_all(venv_bin_path(&context.venv)).unwrap();
 
-    uv_snapshot!(context.filters(), context.python_find().env(EnvVars::VIRTUAL_ENV, context.venv.as_os_str()), @r###"
+    uv_snapshot!(context.filters(), context.python_find().env(EnvVars::VIRTUAL_ENV, context.venv.as_os_str()), @r"
     success: false
     exit_code: 2
     ----- stdout -----
 
     ----- stderr -----
-    error: Failed to inspect Python interpreter from active virtual environment at `.venv/[BIN]/python`
-      Caused by: Python interpreter not found at `[VENV]/[BIN]/python`
-    "###);
+    error: Failed to inspect Python interpreter from active virtual environment at `.venv/[BIN]/[PYTHON]`
+      Caused by: Python interpreter not found at `[VENV]/[BIN]/[PYTHON]`
+    ");
 
     // Unless the virtual environment is not active
     uv_snapshot!(context.filters(), context.python_find(), @r###"
@@ -728,6 +728,57 @@ fn python_find_venv_invalid() {
     "###);
 }
 
+#[test]
+fn python_find_managed() {
+    let context: TestContext = TestContext::new_with_versions(&["3.11", "3.12"])
+        .with_filtered_python_sources()
+        .with_versions_as_managed(&["3.12"]);
+
+    // We find the managed interpreter
+    uv_snapshot!(context.filters(), context.python_find().arg("--managed-python"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    [PYTHON-3.12]
+
+    ----- stderr -----
+    ");
+
+    // Request an interpreter that cannot be satisfied
+    uv_snapshot!(context.filters(), context.python_find().arg("--managed-python").arg("3.11"), @r"
+    success: false
+    exit_code: 2
+    ----- stdout -----
+
+    ----- stderr -----
+    error: No interpreter found for Python 3.11 in virtual environments or managed installations
+    ");
+
+    let context: TestContext = TestContext::new_with_versions(&["3.11", "3.12"])
+        .with_filtered_python_sources()
+        .with_versions_as_managed(&["3.11"]);
+
+    // We find the unmanaged interpreter
+    uv_snapshot!(context.filters(), context.python_find().arg("--no-managed-python"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    [PYTHON-3.12]
+
+    ----- stderr -----
+    ");
+
+    // Request an interpreter that cannot be satisfied
+    uv_snapshot!(context.filters(), context.python_find().arg("--no-managed-python").arg("3.11"), @r"
+    success: false
+    exit_code: 2
+    ----- stdout -----
+
+    ----- stderr -----
+    error: No interpreter found for Python 3.11 in [PYTHON SOURCES]
+    ");
+}
+
 /// See: <https://github.com/astral-sh/uv/issues/11825>
 ///
 /// This test will not succeed on macOS if using a Homebrew provided interpreter. The interpreter
@@ -782,9 +833,10 @@ fn python_required_python_major_minor() {
 #[test]
 fn python_find_script() {
     let context = TestContext::new("3.13")
-        .with_filtered_exe_suffix()
         .with_filtered_virtualenv_bin()
-        .with_filtered_python_names();
+        .with_filtered_python_names()
+        .with_filtered_exe_suffix();
+
     let filters = context
         .filters()
         .into_iter()
@@ -818,7 +870,7 @@ fn python_find_script() {
     success: true
     exit_code: 0
     ----- stdout -----
-    [CACHE_DIR]/environments-v2/[HASHEDNAME]/[BIN]/python
+    [CACHE_DIR]/environments-v2/[HASHEDNAME]/[BIN]/[PYTHON]
 
     ----- stderr -----
     ");
@@ -827,9 +879,9 @@ fn python_find_script() {
 #[test]
 fn python_find_script_no_environment() {
     let context = TestContext::new("3.13")
-        .with_filtered_exe_suffix()
         .with_filtered_virtualenv_bin()
-        .with_filtered_python_names();
+        .with_filtered_python_names()
+        .with_filtered_exe_suffix();
 
     let script = context.temp_dir.child("foo.py");
 
@@ -845,7 +897,7 @@ fn python_find_script_no_environment() {
     success: true
     exit_code: 0
     ----- stdout -----
-    [VENV]/[BIN]/python
+    [VENV]/[BIN]/[PYTHON]
 
     ----- stderr -----
     ");
@@ -872,15 +924,17 @@ fn python_find_script_python_not_found() {
 
     ----- stderr -----
     No interpreter found in [PYTHON SOURCES]
+
+    hint: A managed Python download is available, but Python downloads are set to 'never'
     ");
 }
 
 #[test]
 fn python_find_script_no_such_version() {
     let context = TestContext::new("3.13")
-        .with_filtered_exe_suffix()
         .with_filtered_virtualenv_bin()
         .with_filtered_python_names()
+        .with_filtered_exe_suffix()
         .with_filtered_python_sources();
     let filters = context
         .filters()
