@@ -30,15 +30,49 @@ fn python_install() {
 
     ----- stderr -----
     Installed Python 3.13.5 in [TIME]
-     + cpython-3.13.5-[PLATFORM]
+     + cpython-3.13.5-[PLATFORM] (python3.13)
     ");
 
     let bin_python = context
         .bin_dir
         .child(format!("python3.13{}", std::env::consts::EXE_SUFFIX));
 
-    // The executable should not be installed in the bin directory (requires preview)
-    bin_python.assert(predicate::path::missing());
+    // The executable should be installed in the bin directory
+    bin_python.assert(predicate::path::exists());
+
+    // On Unix, it should be a link
+    #[cfg(unix)]
+    bin_python.assert(predicate::path::is_symlink());
+
+    // The link should be a path to the binary
+    if cfg!(unix) {
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python), @"[TEMP_DIR]/managed/cpython-3.13.5-[PLATFORM]/bin/python3.13"
+            );
+        });
+    } else if cfg!(windows) {
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python), @"[TEMP_DIR]/managed/cpython-3.13.5-[PLATFORM]/python"
+            );
+        });
+    }
+
+    // The executable should "work"
+    uv_snapshot!(context.filters(), Command::new(bin_python.as_os_str())
+        .arg("-c").arg("import subprocess; print('hello world')"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    hello world
+
+    ----- stderr -----
+    "###);
 
     // Should be a no-op when already installed
     uv_snapshot!(context.filters(), context.python_install(), @r###"
@@ -67,9 +101,12 @@ fn python_install() {
 
     ----- stderr -----
     Installed Python 3.13.5 in [TIME]
-     ~ cpython-3.13.5-[PLATFORM]
+     ~ cpython-3.13.5-[PLATFORM] (python3.13)
     ");
 
+    // The executable should still be present in the bin directory
+    bin_python.assert(predicate::path::exists());
+
     // Uninstallation requires an argument
     uv_snapshot!(context.filters(), context.python_uninstall(), @r###"
     success: false
@@ -93,8 +130,11 @@ fn python_install() {
     ----- stderr -----
     Searching for Python versions matching: Python 3.13
     Uninstalled Python 3.13.5 in [TIME]
-     - cpython-3.13.5-[PLATFORM]
+     - cpython-3.13.5-[PLATFORM] (python3.13)
     ");
+
+    // The executable should be removed
+    bin_python.assert(predicate::path::missing());
 }
 
 #[test]
@@ -112,8 +152,8 @@ fn python_reinstall() {
 
     ----- stderr -----
     Installed 2 versions in [TIME]
-     + cpython-3.12.11-[PLATFORM]
-     + cpython-3.13.5-[PLATFORM]
+     + cpython-3.12.11-[PLATFORM] (python3.12)
+     + cpython-3.13.5-[PLATFORM] (python3.13)
     ");
 
     // Reinstall a single version
@@ -124,7 +164,7 @@ fn python_reinstall() {
 
     ----- stderr -----
     Installed Python 3.13.5 in [TIME]
-     ~ cpython-3.13.5-[PLATFORM]
+     ~ cpython-3.13.5-[PLATFORM] (python3.13)
     ");
 
     // Reinstall multiple versions
@@ -135,8 +175,8 @@ fn python_reinstall() {
 
     ----- stderr -----
     Installed 2 versions in [TIME]
-     ~ cpython-3.12.11-[PLATFORM]
-     ~ cpython-3.13.5-[PLATFORM]
+     ~ cpython-3.12.11-[PLATFORM] (python3.12)
+     ~ cpython-3.13.5-[PLATFORM] (python3.13)
     ");
 
     // Reinstalling a version that is not installed should also work
@@ -147,7 +187,7 @@ fn python_reinstall() {
 
     ----- stderr -----
     Installed Python 3.11.13 in [TIME]
-     + cpython-3.11.13-[PLATFORM]
+     + cpython-3.11.13-[PLATFORM] (python3.11)
     ");
 }
 
@@ -167,7 +207,7 @@ fn python_reinstall_patch() {
     ----- stderr -----
     Installed 2 versions in [TIME]
      + cpython-3.12.6-[PLATFORM]
-     + cpython-3.12.7-[PLATFORM]
+     + cpython-3.12.7-[PLATFORM] (python3.12)
     ");
 
     // Reinstall all "3.12" versions
@@ -180,7 +220,7 @@ fn python_reinstall_patch() {
 
     ----- stderr -----
     Installed Python 3.12.11 in [TIME]
-     + cpython-3.12.11-[PLATFORM]
+     + cpython-3.12.11-[PLATFORM] (python3.12)
     ");
 }
 
@@ -196,14 +236,16 @@ fn python_install_automatic() {
     uv_snapshot!(context.filters(), context.run()
         .env_remove("VIRTUAL_ENV")
         .arg("--no-python-downloads")
-        .arg("python").arg("-c").arg("import sys; print(sys.version_info[:2])"), @r###"
+        .arg("python").arg("-c").arg("import sys; print(sys.version_info[:2])"), @r"
     success: false
     exit_code: 2
     ----- stdout -----
 
     ----- stderr -----
     error: No interpreter found in [PYTHON SOURCES]
-    "###);
+
+    hint: A managed Python download is available, but Python downloads are set to 'never'
+    ");
 
     // Otherwise, we should fetch the latest Python version
     uv_snapshot!(context.filters(), context.run()
@@ -326,6 +368,208 @@ fn regression_cpython() {
     "###);
 }
 
+#[test]
+fn python_install_force() {
+    let context: TestContext = TestContext::new_with_versions(&[])
+        .with_filtered_python_keys()
+        .with_filtered_exe_suffix()
+        .with_managed_python_dirs();
+
+    // Install the latest version
+    uv_snapshot!(context.filters(), context.python_install(), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Installed Python 3.13.5 in [TIME]
+     + cpython-3.13.5-[PLATFORM] (python3.13)
+    ");
+
+    let bin_python = context
+        .bin_dir
+        .child(format!("python3.13{}", std::env::consts::EXE_SUFFIX));
+
+    // You can force replacement of the executables
+    uv_snapshot!(context.filters(), context.python_install().arg("--force"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Installed Python 3.13.5 in [TIME]
+     + cpython-3.13.5-[PLATFORM] (python3.13)
+    ");
+
+    // The executable should still be present in the bin directory
+    bin_python.assert(predicate::path::exists());
+
+    // If an unmanaged executable is present, `--force` is required
+    fs_err::remove_file(bin_python.path()).unwrap();
+    bin_python.touch().unwrap();
+
+    uv_snapshot!(context.filters(), context.python_install().arg("3.13"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: Failed to install executable for cpython-3.13.5-[PLATFORM]
+      Caused by: Executable already exists at `[BIN]/python3.13` but is not managed by uv; use `--force` to replace it
+    ");
+
+    uv_snapshot!(context.filters(), context.python_install().arg("--force").arg("3.13"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Installed Python 3.13.5 in [TIME]
+     + cpython-3.13.5-[PLATFORM] (python3.13)
+    ");
+
+    bin_python.assert(predicate::path::exists());
+}
+
+#[test]
+fn python_install_minor() {
+    let context: TestContext = TestContext::new_with_versions(&[])
+        .with_filtered_python_keys()
+        .with_filtered_exe_suffix()
+        .with_managed_python_dirs();
+
+    // Install a minor version
+    uv_snapshot!(context.filters(), context.python_install().arg("3.11"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Installed Python 3.11.13 in [TIME]
+     + cpython-3.11.13-[PLATFORM] (python3.11)
+    ");
+
+    let bin_python = context
+        .bin_dir
+        .child(format!("python3.11{}", std::env::consts::EXE_SUFFIX));
+
+    // The executable should be installed in the bin directory
+    bin_python.assert(predicate::path::exists());
+
+    // It should be a link to the minor version
+    if cfg!(unix) {
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python), @"[TEMP_DIR]/managed/cpython-3.11.13-[PLATFORM]/bin/python3.11"
+            );
+        });
+    } else if cfg!(windows) {
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python), @"[TEMP_DIR]/managed/cpython-3.11.13-[PLATFORM]/python"
+            );
+        });
+    }
+
+    uv_snapshot!(context.filters(), context.python_uninstall().arg("3.11"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Searching for Python versions matching: Python 3.11
+    Uninstalled Python 3.11.13 in [TIME]
+     - cpython-3.11.13-[PLATFORM] (python3.11)
+    ");
+
+    // The executable should be removed
+    bin_python.assert(predicate::path::missing());
+}
+
+#[test]
+fn python_install_multiple_patch() {
+    let context: TestContext = TestContext::new_with_versions(&[])
+        .with_filtered_python_keys()
+        .with_filtered_exe_suffix()
+        .with_managed_python_dirs();
+
+    // Install multiple patch versions
+    uv_snapshot!(context.filters(), context.python_install().arg("3.12.8").arg("3.12.6"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Installed 2 versions in [TIME]
+     + cpython-3.12.6-[PLATFORM]
+     + cpython-3.12.8-[PLATFORM] (python3.12)
+    ");
+
+    let bin_python = context
+        .bin_dir
+        .child(format!("python3.12{}", std::env::consts::EXE_SUFFIX));
+
+    // The executable should be installed in the bin directory
+    bin_python.assert(predicate::path::exists());
+
+    // The link should resolve to the newer patch version
+    if cfg!(unix) {
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                canonicalize_link_path(&bin_python), @"[TEMP_DIR]/managed/cpython-3.12.8-[PLATFORM]/bin/python3.12"
+            );
+        });
+    } else if cfg!(windows) {
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                canonicalize_link_path(&bin_python), @"[TEMP_DIR]/managed/cpython-3.12.8-[PLATFORM]/python"
+            );
+        });
+    }
+
+    uv_snapshot!(context.filters(), context.python_uninstall().arg("3.12.8"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Searching for Python versions matching: Python 3.12.8
+    Uninstalled Python 3.12.8 in [TIME]
+     - cpython-3.12.8-[PLATFORM] (python3.12)
+    ");
+
+    // TODO(zanieb): This behavior is not implemented yet
+    // // The executable should be installed in the bin directory
+    // bin_python.assert(predicate::path::exists());
+
+    // // When the version is removed, the link should point to the other patch version
+    // if cfg!(unix) {
+    //     insta::with_settings!({
+    //         filters => context.filters(),
+    //     }, {
+    //         insta::assert_snapshot!(
+    //             canonicalize_link_path(&bin_python), @"[TEMP_DIR]/managed/cpython-3.12.6-[PLATFORM]/bin/python3.12"
+    //         );
+    //     });
+    // } else if cfg!(windows) {
+    //     insta::with_settings!({
+    //         filters => context.filters(),
+    //     }, {
+    //         insta::assert_snapshot!(
+    //             canonicalize_link_path(&bin_python), @"[TEMP_DIR]/managed/cpython-3.12.6-[PLATFORM]/python"
+    //         );
+    //     });
+    // }
+}
+
 #[test]
 fn python_install_preview() {
     let context: TestContext = TestContext::new_with_versions(&[])
@@ -428,14 +672,50 @@ fn python_install_preview() {
     bin_python.touch().unwrap();
 
     uv_snapshot!(context.filters(), context.python_install().arg("--preview").arg("3.13"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: Failed to install executable for cpython-3.13.5-[PLATFORM]
+      Caused by: Executable already exists at `[BIN]/python3.13` but is not managed by uv; use `--force` to replace it
+    ");
+
+    // With `--bin`, this should error instead of warn
+    uv_snapshot!(context.filters(), context.python_install().arg("--preview").arg("--bin").arg("3.13"), @r"
     success: false
     exit_code: 1
     ----- stdout -----
 
     ----- stderr -----
-    error: Failed to install cpython-3.13.5-[PLATFORM]
+    error: Failed to install executable for cpython-3.13.5-[PLATFORM]
       Caused by: Executable already exists at `[BIN]/python3.13` but is not managed by uv; use `--force` to replace it
     ");
+    uv_snapshot!(context.filters(), context.python_install().arg("--preview").arg("3.13").env(EnvVars::UV_PYTHON_INSTALL_BIN, "1"), @r"
+    success: false
+    exit_code: 1
+    ----- stdout -----
+
+    ----- stderr -----
+    error: Failed to install executable for cpython-3.13.5-[PLATFORM]
+      Caused by: Executable already exists at `[BIN]/python3.13` but is not managed by uv; use `--force` to replace it
+    ");
+
+    // With `--no-bin`, this should be silent
+    uv_snapshot!(context.filters(), context.python_install().arg("--preview").arg("--no-bin").arg("3.13"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    ");
+    uv_snapshot!(context.filters(), context.python_install().arg("--preview").arg("3.13").env(EnvVars::UV_PYTHON_INSTALL_BIN, "0"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    ");
 
     uv_snapshot!(context.filters(), context.python_install().arg("--preview").arg("--force").arg("3.13"), @r"
     success: true
@@ -563,6 +843,52 @@ fn python_install_preview() {
     }
 }
 
+#[test]
+fn python_install_preview_no_bin() {
+    let context: TestContext = TestContext::new_with_versions(&[])
+        .with_filtered_python_keys()
+        .with_filtered_exe_suffix()
+        .with_managed_python_dirs();
+
+    // Install the latest version
+    uv_snapshot!(context.filters(), context.python_install().arg("--preview").arg("--no-bin"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Installed Python 3.13.5 in [TIME]
+     + cpython-3.13.5-[PLATFORM]
+    ");
+
+    let bin_python = context
+        .bin_dir
+        .child(format!("python3.13{}", std::env::consts::EXE_SUFFIX));
+
+    // The executable should not be installed in the bin directory
+    bin_python.assert(predicate::path::missing());
+
+    uv_snapshot!(context.filters(), context.python_install().arg("--preview").arg("--no-bin").arg("--default"), @r"
+    success: false
+    exit_code: 2
+    ----- stdout -----
+
+    ----- stderr -----
+    error: the argument '--no-bin' cannot be used with '--default'
+
+    Usage: uv python install --no-bin --install-dir <INSTALL_DIR> [TARGETS]...
+
+    For more information, try '--help'.
+    ");
+
+    let bin_python = context
+        .bin_dir
+        .child(format!("python{}", std::env::consts::EXE_SUFFIX));
+
+    // The executable should not be installed in the bin directory
+    bin_python.assert(predicate::path::missing());
+}
+
 #[test]
 fn python_install_preview_upgrade() {
     let context = TestContext::new_with_versions(&[])
@@ -575,7 +901,7 @@ fn python_install_preview_upgrade() {
         .child(format!("python3.12{}", std::env::consts::EXE_SUFFIX));
 
     // Install 3.12.5
-    uv_snapshot!(context.filters(), context.python_install().arg("--preview").arg("3.12.5"), @r###"
+    uv_snapshot!(context.filters(), context.python_install().arg("--preview").arg("3.12.5"), @r"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -583,7 +909,7 @@ fn python_install_preview_upgrade() {
     ----- stderr -----
     Installed Python 3.12.5 in [TIME]
      + cpython-3.12.5-[PLATFORM] (python3.12)
-    "###);
+    ");
 
     // Installing with a patch version should cause the link to be to the patch installation.
     if cfg!(unix) {
@@ -769,7 +1095,7 @@ fn python_install_freethreaded() {
 
     ----- stderr -----
     Installed Python 3.13.5 in [TIME]
-     + cpython-3.13.5-[PLATFORM]
+     + cpython-3.13.5-[PLATFORM] (python3.13)
     ");
 
     // Should not work with older Python versions
@@ -791,7 +1117,7 @@ fn python_install_freethreaded() {
     Searching for Python installations
     Uninstalled 2 versions in [TIME]
      - cpython-3.13.5+freethreaded-[PLATFORM] (python3.13t)
-     - cpython-3.13.5-[PLATFORM]
+     - cpython-3.13.5-[PLATFORM] (python3.13)
     ");
 }
 
@@ -852,15 +1178,243 @@ fn python_install_default() {
         .bin_dir
         .child(format!("python{}", std::env::consts::EXE_SUFFIX));
 
-    // `--preview` is required for `--default`
-    uv_snapshot!(context.filters(), context.python_install().arg("--default"), @r###"
-    success: false
-    exit_code: 1
+    // Install a specific version
+    uv_snapshot!(context.filters(), context.python_install().arg("3.13"), @r"
+    success: true
+    exit_code: 0
     ----- stdout -----
 
     ----- stderr -----
-    The `--default` flag is only available in preview mode; add the `--preview` flag to use `--default`
-    "###);
+    Installed Python 3.13.5 in [TIME]
+     + cpython-3.13.5-[PLATFORM] (python3.13)
+    ");
+
+    // Only the minor versioned executable should be installed
+    bin_python_minor_13.assert(predicate::path::exists());
+    bin_python_major.assert(predicate::path::missing());
+    bin_python_default.assert(predicate::path::missing());
+
+    // Install again, with `--default`
+    uv_snapshot!(context.filters(), context.python_install().arg("--default").arg("3.13"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: The `--default` option is experimental and may change without warning. Pass `--preview` to disable this warning
+    Installed Python 3.13.5 in [TIME]
+     + cpython-3.13.5-[PLATFORM] (python, python3)
+    ");
+
+    // Now all the executables should be installed
+    bin_python_minor_13.assert(predicate::path::exists());
+    bin_python_major.assert(predicate::path::exists());
+    bin_python_default.assert(predicate::path::exists());
+
+    // Uninstall
+    uv_snapshot!(context.filters(), context.python_uninstall().arg("--all"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Searching for Python installations
+    Uninstalled Python 3.13.5 in [TIME]
+     - cpython-3.13.5-[PLATFORM] (python, python3, python3.13)
+    ");
+
+    // The executables should be removed
+    bin_python_minor_13.assert(predicate::path::missing());
+    bin_python_major.assert(predicate::path::missing());
+    bin_python_default.assert(predicate::path::missing());
+
+    // Install the latest version, i.e., a "default install"
+    uv_snapshot!(context.filters(), context.python_install().arg("--default"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: The `--default` option is experimental and may change without warning. Pass `--preview` to disable this warning
+    Installed Python 3.13.5 in [TIME]
+     + cpython-3.13.5-[PLATFORM] (python, python3, python3.13)
+    ");
+
+    // Since it's a default install, we should include all of the executables
+    bin_python_minor_13.assert(predicate::path::exists());
+    bin_python_major.assert(predicate::path::exists());
+    bin_python_default.assert(predicate::path::exists());
+
+    // And 3.13 should be the default
+    if cfg!(unix) {
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python_major), @"[TEMP_DIR]/managed/cpython-3.13.5-[PLATFORM]/bin/python3.13"
+            );
+        });
+
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python_minor_13), @"[TEMP_DIR]/managed/cpython-3.13.5-[PLATFORM]/bin/python3.13"
+            );
+        });
+
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python_default), @"[TEMP_DIR]/managed/cpython-3.13.5-[PLATFORM]/bin/python3.13"
+            );
+        });
+    } else if cfg!(windows) {
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python_major), @"[TEMP_DIR]/managed/cpython-3.13.5-[PLATFORM]/python"
+            );
+        });
+
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python_minor_13), @"[TEMP_DIR]/managed/cpython-3.13.5-[PLATFORM]/python"
+            );
+        });
+
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python_default), @"[TEMP_DIR]/managed/cpython-3.13.5-[PLATFORM]/python"
+            );
+        });
+    }
+
+    // Uninstall again
+    uv_snapshot!(context.filters(), context.python_uninstall().arg("3.13"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Searching for Python versions matching: Python 3.13
+    Uninstalled Python 3.13.5 in [TIME]
+     - cpython-3.13.5-[PLATFORM] (python, python3, python3.13)
+    ");
+
+    // We should remove all the executables
+    bin_python_minor_13.assert(predicate::path::missing());
+    bin_python_major.assert(predicate::path::missing());
+    bin_python_default.assert(predicate::path::missing());
+
+    // Install multiple versions, with the `--default` flag
+    uv_snapshot!(context.filters(), context.python_install().arg("3.12").arg("3.13").arg("--default"), @r"
+    success: false
+    exit_code: 2
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: The `--default` option is experimental and may change without warning. Pass `--preview` to disable this warning
+    error: The `--default` flag cannot be used with multiple targets
+    ");
+
+    // Install 3.12 as a new default
+    uv_snapshot!(context.filters(), context.python_install().arg("3.12").arg("--default"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: The `--default` option is experimental and may change without warning. Pass `--preview` to disable this warning
+    Installed Python 3.12.11 in [TIME]
+     + cpython-3.12.11-[PLATFORM] (python, python3, python3.12)
+    ");
+
+    let bin_python_minor_12 = context
+        .bin_dir
+        .child(format!("python3.12{}", std::env::consts::EXE_SUFFIX));
+
+    // All the executables should exist
+    bin_python_minor_12.assert(predicate::path::exists());
+    bin_python_major.assert(predicate::path::exists());
+    bin_python_default.assert(predicate::path::exists());
+
+    // And 3.12 should be the default
+    if cfg!(unix) {
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python_major), @"[TEMP_DIR]/managed/cpython-3.12.11-[PLATFORM]/bin/python3.12"
+            );
+        });
+
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python_minor_12), @"[TEMP_DIR]/managed/cpython-3.12.11-[PLATFORM]/bin/python3.12"
+            );
+        });
+
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python_default), @"[TEMP_DIR]/managed/cpython-3.12.11-[PLATFORM]/bin/python3.12"
+            );
+        });
+    } else {
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python_major), @"[TEMP_DIR]/managed/cpython-3.12.11-[PLATFORM]/python"
+            );
+        });
+
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python_minor_12), @"[TEMP_DIR]/managed/cpython-3.12.11-[PLATFORM]/python"
+            );
+        });
+
+        insta::with_settings!({
+            filters => context.filters(),
+        }, {
+            insta::assert_snapshot!(
+                read_link(&bin_python_default), @"[TEMP_DIR]/managed/cpython-3.12.11-[PLATFORM]/python"
+            );
+        });
+    }
+}
+
+#[test]
+fn python_install_default_preview() {
+    let context: TestContext = TestContext::new_with_versions(&[])
+        .with_filtered_python_keys()
+        .with_filtered_exe_suffix()
+        .with_managed_python_dirs();
+
+    let bin_python_minor_13 = context
+        .bin_dir
+        .child(format!("python3.13{}", std::env::consts::EXE_SUFFIX));
+
+    let bin_python_major = context
+        .bin_dir
+        .child(format!("python3{}", std::env::consts::EXE_SUFFIX));
+
+    let bin_python_default = context
+        .bin_dir
+        .child(format!("python{}", std::env::consts::EXE_SUFFIX));
 
     // Install a specific version
     uv_snapshot!(context.filters(), context.python_install().arg("--preview").arg("3.13"), @r"
@@ -1258,7 +1812,7 @@ fn python_install_unknown() {
 
 #[cfg(unix)]
 #[test]
-fn python_install_preview_broken_link() {
+fn python_install_broken_link() {
     use assert_fs::prelude::PathCreateDir;
     use fs_err::os::unix::fs::symlink;
 
@@ -1274,7 +1828,7 @@ fn python_install_preview_broken_link() {
     symlink(context.temp_dir.join("does-not-exist"), &bin_python).unwrap();
 
     // Install
-    uv_snapshot!(context.filters(), context.python_install().arg("--preview").arg("3.13"), @r"
+    uv_snapshot!(context.filters(), context.python_install().arg("3.13"), @r"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -1309,7 +1863,7 @@ fn python_install_default_from_env() {
 
     ----- stderr -----
     Installed Python 3.12.11 in [TIME]
-     + cpython-3.12.11-[PLATFORM]
+     + cpython-3.12.11-[PLATFORM] (python3.12)
     ");
 
     // But prefer explicit requests
@@ -1320,7 +1874,7 @@ fn python_install_default_from_env() {
 
     ----- stderr -----
     Installed Python 3.11.13 in [TIME]
-     + cpython-3.11.13-[PLATFORM]
+     + cpython-3.11.13-[PLATFORM] (python3.11)
     ");
 
     // We should ignore `UV_PYTHON` here and complain there is not a target
@@ -1347,8 +1901,8 @@ fn python_install_default_from_env() {
     ----- stderr -----
     Searching for Python installations
     Uninstalled 2 versions in [TIME]
-     - cpython-3.11.13-[PLATFORM]
-     - cpython-3.12.11-[PLATFORM]
+     - cpython-3.11.13-[PLATFORM] (python3.11)
+     - cpython-3.12.11-[PLATFORM] (python3.12)
     ");
 
     // Uninstall with no targets should error
@@ -1431,10 +1985,8 @@ fn python_install_patch_dylib() {
 fn python_install_314() {
     let context: TestContext = TestContext::new_with_versions(&[])
         .with_filtered_python_keys()
-        .with_filtered_exe_suffix()
         .with_managed_python_dirs()
-        .with_filtered_python_names()
-        .with_filtered_python_install_bin();
+        .with_filtered_exe_suffix();
 
     // Install 3.14
     // For now, this provides test coverage of pre-release handling
@@ -1445,7 +1997,7 @@ fn python_install_314() {
 
     ----- stderr -----
     Installed Python 3.14.0b4 in [TIME]
-     + cpython-3.14.0b4-[PLATFORM]
+     + cpython-3.14.0b4-[PLATFORM] (python3.14)
     ");
 
     // Install a specific pre-release
@@ -1459,13 +2011,24 @@ fn python_install_314() {
      + cpython-3.14.0a4-[PLATFORM]
     ");
 
+    // Add name filtering for the `find` tests, we avoid it in `install` tests because it clobbers
+    // the version suffixes which matter in the install logs
+    let filters = context
+        .filters()
+        .iter()
+        .map(|(a, b)| ((*a).to_string(), (*b).to_string()))
+        .collect::<Vec<_>>();
+    let context = context
+        .with_filtered_python_install_bin()
+        .with_filtered_python_names();
+
     // We should be able to find this version without opt-in, because there is no stable release
     // installed
     uv_snapshot!(context.filters(), context.python_find().arg("3.14"), @r"
     success: true
     exit_code: 0
     ----- stdout -----
-    [TEMP_DIR]/managed/cpython-3.14.0b4-[PLATFORM]/[INSTALL-BIN]/python
+    [TEMP_DIR]/managed/cpython-3.14.0b4-[PLATFORM]/[INSTALL-BIN]/[PYTHON]
 
     ----- stderr -----
     ");
@@ -1475,7 +2038,7 @@ fn python_install_314() {
     success: true
     exit_code: 0
     ----- stdout -----
-    [TEMP_DIR]/managed/cpython-3.14.0b4-[PLATFORM]/[INSTALL-BIN]/python
+    [TEMP_DIR]/managed/cpython-3.14.0b4-[PLATFORM]/[INSTALL-BIN]/[PYTHON]
 
     ----- stderr -----
     ");
@@ -1484,27 +2047,27 @@ fn python_install_314() {
     success: true
     exit_code: 0
     ----- stdout -----
-    [TEMP_DIR]/managed/cpython-3.14.0b4-[PLATFORM]/[INSTALL-BIN]/python
+    [TEMP_DIR]/managed/cpython-3.14.0b4-[PLATFORM]/[INSTALL-BIN]/[PYTHON]
 
     ----- stderr -----
     ");
 
     // If we install a stable version, that should be preferred though
-    uv_snapshot!(context.filters(), context.python_install().arg("3.13"), @r"
+    uv_snapshot!(filters, context.python_install().arg("3.13"), @r"
     success: true
     exit_code: 0
     ----- stdout -----
 
     ----- stderr -----
     Installed Python 3.13.5 in [TIME]
-     + cpython-3.13.5-[PLATFORM]
+     + cpython-3.13.5-[PLATFORM] (python3.13)
     ");
 
     uv_snapshot!(context.filters(), context.python_find().arg("3"), @r"
     success: true
     exit_code: 0
     ----- stdout -----
-    [TEMP_DIR]/managed/cpython-3.13.5-[PLATFORM]/[INSTALL-BIN]/python
+    [TEMP_DIR]/managed/cpython-3.13.5-[PLATFORM]/[INSTALL-BIN]/[PYTHON]
 
     ----- stderr -----
     ");
@@ -1537,15 +2100,15 @@ fn python_install_cached() {
 
     ----- stderr -----
     Installed Python 3.13.5 in [TIME]
-     + cpython-3.13.5-[PLATFORM]
+     + cpython-3.13.5-[PLATFORM] (python3.13)
     ");
 
     let bin_python = context
         .bin_dir
         .child(format!("python3.13{}", std::env::consts::EXE_SUFFIX));
 
-    // The executable should not be installed in the bin directory (requires preview)
-    bin_python.assert(predicate::path::missing());
+    // The executable should be installed in the bin directory
+    bin_python.assert(predicate::path::exists());
 
     // Should be a no-op when already installed
     uv_snapshot!(context.filters(), context
@@ -1567,7 +2130,7 @@ fn python_install_cached() {
     ----- stderr -----
     Searching for Python versions matching: Python 3.13
     Uninstalled Python 3.13.5 in [TIME]
-     - cpython-3.13.5-[PLATFORM]
+     - cpython-3.13.5-[PLATFORM] (python3.13)
     ");
 
     // The cached archive can be installed offline
@@ -1581,7 +2144,7 @@ fn python_install_cached() {
 
     ----- stderr -----
     Installed Python 3.13.5 in [TIME]
-     + cpython-3.13.5-[PLATFORM]
+     + cpython-3.13.5-[PLATFORM] (python3.13)
     ");
 
     // 3.12 isn't cached, so it can't be installed
@@ -1630,7 +2193,7 @@ fn python_install_emulated_macos() {
 
     ----- stderr -----
     Installed Python 3.13.5 in [TIME]
-     + cpython-3.13.5-macos-x86_64-none
+     + cpython-3.13.5-macos-x86_64-none (python3.13)
     ");
 
     // It should be discoverable with `uv python find`
