@@ -164,7 +164,7 @@ fn python_pin() {
     // (skip on Windows because the snapshot is different and the behavior is not platform dependent)
     #[cfg(unix)]
     {
-        uv_snapshot!(context.filters(), context.python_pin().arg("pypy"), @r###"
+        uv_snapshot!(context.filters(), context.python_pin().arg("pypy"), @r"
         success: true
         exit_code: 0
         ----- stdout -----
@@ -172,7 +172,7 @@ fn python_pin() {
 
         ----- stderr -----
         warning: No interpreter found for PyPy in managed installations or search path
-        "###);
+        ");
 
         let python_version = context.read(PYTHON_VERSION_FILENAME);
         assert_snapshot!(python_version, @r###"
@@ -361,7 +361,7 @@ fn python_pin_global_creates_parent_dirs() {
 fn python_pin_no_python() {
     let context: TestContext = TestContext::new_with_versions(&[]);
 
-    uv_snapshot!(context.filters(), context.python_pin().arg("3.12"), @r###"
+    uv_snapshot!(context.filters(), context.python_pin().arg("3.12"), @r"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -369,7 +369,7 @@ fn python_pin_no_python() {
 
     ----- stderr -----
     warning: No interpreter found for Python 3.12 in managed installations or search path
-    "###);
+    ");
 }
 
 #[test]
@@ -448,7 +448,7 @@ fn python_pin_compatible_with_requires_python() -> Result<()> {
     "###);
 
     // Request a version that is compatible and uses a Python variant
-    uv_snapshot!(context.filters(), context.python_pin().arg("3.13t"), @r###"
+    uv_snapshot!(context.filters(), context.python_pin().arg("3.13t"), @r"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -456,7 +456,7 @@ fn python_pin_compatible_with_requires_python() -> Result<()> {
 
     ----- stderr -----
     warning: No interpreter found for Python 3.13t in [PYTHON SOURCES]
-    "###);
+    ");
 
     // Request a implementation version that is compatible
     uv_snapshot!(context.filters(), context.python_pin().arg("cpython@3.11"), @r###"
@@ -587,27 +587,17 @@ fn warning_pinned_python_version_not_installed() -> Result<()> {
 /// We do need a Python interpreter for `--resolved` pins
 #[test]
 fn python_pin_resolve_no_python() {
-    let context: TestContext = TestContext::new_with_versions(&[]);
+    let context: TestContext = TestContext::new_with_versions(&[]).with_filtered_python_sources();
+    uv_snapshot!(context.filters(), context.python_pin().arg("--resolved").arg("3.12"), @r"
+    success: false
+    exit_code: 2
+    ----- stdout -----
 
-    if cfg!(windows) {
-        uv_snapshot!(context.filters(), context.python_pin().arg("--resolved").arg("3.12"), @r###"
-        success: false
-        exit_code: 2
-        ----- stdout -----
+    ----- stderr -----
+    error: No interpreter found for Python 3.12 in [PYTHON SOURCES]
 
-        ----- stderr -----
-        error: No interpreter found for Python 3.12 in managed installations, search path, or registry
-        "###);
-    } else {
-        uv_snapshot!(context.filters(), context.python_pin().arg("--resolved").arg("3.12"), @r###"
-        success: false
-        exit_code: 2
-        ----- stdout -----
-
-        ----- stderr -----
-        error: No interpreter found for Python 3.12 in managed installations or search path
-        "###);
-    }
+    hint: A managed Python download is available for Python 3.12, but Python downloads are set to 'never'
+    ");
 }
 
 #[test]
@@ -741,14 +731,16 @@ fn python_pin_resolve() {
     // Request an implementation that is not installed
     // (skip on Windows because the snapshot is different and the behavior is not platform dependent)
     #[cfg(unix)]
-    uv_snapshot!(context.filters(), context.python_pin().arg("--resolved").arg("pypy"), @r###"
+    uv_snapshot!(context.filters(), context.python_pin().arg("--resolved").arg("pypy"), @r"
     success: false
     exit_code: 2
     ----- stdout -----
 
     ----- stderr -----
     error: No interpreter found for PyPy in managed installations or search path
-    "###);
+
+    hint: A managed Python download is available for PyPy, but Python downloads are set to 'never'
+    ");
 
     let python_version = context.read(PYTHON_VERSION_FILENAME);
     insta::with_settings!({
@@ -855,7 +847,7 @@ fn python_pin_rm() {
     error: No Python version file found
     ");
 
-    // Remove the local pin
+    // Create and remove a local pin
     context.python_pin().arg("3.12").assert().success();
     uv_snapshot!(context.filters(), context.python_pin().arg("--rm"), @r"
     success: true
@@ -892,12 +884,41 @@ fn python_pin_rm() {
         .arg("--global")
         .assert()
         .success();
+
     uv_snapshot!(context.filters(), context.python_pin().arg("--rm").arg("--global"), @r"
     success: true
     exit_code: 0
     ----- stdout -----
-    Removed Python version file at `[UV_USER_CONFIG_DIR]/.python-version`
+    Removed global Python pin at `[UV_USER_CONFIG_DIR]/.python-version`
 
     ----- stderr -----
     ");
+
+    // Add the global pin again
+    context
+        .python_pin()
+        .arg("3.12")
+        .arg("--global")
+        .assert()
+        .success();
+
+    // Remove the local pin
+    uv_snapshot!(context.filters(), context.python_pin().arg("--rm"), @r"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    Removed Python version file at `.python-version`
+
+    ----- stderr -----
+    ");
+
+    // The global pin should not be removed without `--global`
+    uv_snapshot!(context.filters(), context.python_pin().arg("--rm"), @r"
+    success: false
+    exit_code: 2
+    ----- stdout -----
+
+    ----- stderr -----
+    error: No Python version file found; use `--rm --global` to remove the global pin
+    ");
 }
